# coding=utf-8
"""
Unit tests for HuaweiDNSProvider

@author: GitHub Copilot
"""

from base_test import BaseProviderTestCase, unittest, patch
from ddns.provider.huaweidns import HuaweiDNSProvider


class TestHuaweiDNSProvider(BaseProviderTestCase):
    """Test cases for HuaweiDNSProvider"""

    def setUp(self):
        """Set up test fixtures"""
        super(TestHuaweiDNSProvider, self).setUp()
        self.authid = "test_access_key"
        self.token = "test_secret_key"
        self.provider = HuaweiDNSProvider(self.authid, self.token)

        # Mock strftime for all tests
        self.strftime_patcher = patch("ddns.provider.huaweidns.strftime")
        self.mock_strftime = self.strftime_patcher.start()
        self.mock_strftime.return_value = "20230101T120000Z"

    def tearDown(self):
        """Clean up test fixtures"""
        self.strftime_patcher.stop()
        super(TestHuaweiDNSProvider, self).tearDown()

    def test_class_constants(self):
        """Test HuaweiDNSProvider class constants"""
        self.assertEqual(HuaweiDNSProvider.endpoint, "https://dns.myhuaweicloud.com")
        self.assertEqual(HuaweiDNSProvider.content_type, "application/json")
        self.assertTrue(HuaweiDNSProvider.decode_response)
        self.assertEqual(HuaweiDNSProvider.algorithm, "SDK-HMAC-SHA256")

    def test_init_with_basic_config(self):
        """Test HuaweiDNSProvider initialization with basic configuration"""
        self.assertEqual(self.provider.id, self.authid)
        self.assertEqual(self.provider.token, self.token)
        self.assertEqual(self.provider.endpoint, "https://dns.myhuaweicloud.com")

    def test_request_get_method(self):
        """Test _request method with GET method"""
        with patch.object(self.provider, "_http") as mock_http:
            mock_http.return_value = {"zones": []}

            result = self.provider._request("GET", "/v2/zones", name="example.com", limit=500)

            mock_http.assert_called_once()
            self.assertEqual(result, {"zones": []})

    def test_request_post_method(self):
        """Test _request method with POST method"""
        with patch.object(self.provider, "_http") as mock_http:
            mock_http.return_value = {"id": "record123"}

            result = self.provider._request(
                "POST", "/v2.1/zones/zone123/recordsets", name="www.example.com", type="A", records=["1.2.3.4"]
            )

            mock_http.assert_called_once()
            self.assertEqual(result, {"id": "record123"})

    def test_request_filters_none_params(self):
        """Test _request method filters out None parameters"""
        with patch.object(self.provider, "_http") as mock_http:
            mock_http.return_value = {"zones": []}

            self.provider._request("GET", "/v2/zones", name="example.com", limit=None, type=None)

            # Verify that _http was called (None params should be filtered)
            mock_http.assert_called_once()

    def test_query_zone_id_success(self):
        """Test _query_zone_id method with successful response"""
        with patch.object(self.provider, "_request") as mock_request:
            mock_request.return_value = {
                "zones": [{"id": "zone123", "name": "example.com."}, {"id": "zone456", "name": "another.com."}]
            }

            result = self.provider._query_zone_id("example.com")

            mock_request.assert_called_once_with(
                "GET", "/v2/zones", search_mode="equal", limit=500, name="example.com."
            )
            self.assertEqual(result, "zone123")

    def test_query_zone_id_with_trailing_dot(self):
        """Test _query_zone_id method with domain already having trailing dot"""
        with patch.object(self.provider, "_request") as mock_request:
            mock_request.return_value = {"zones": [{"id": "zone123", "name": "example.com."}]}

            result = self.provider._query_zone_id("example.com.")

            mock_request.assert_called_once_with(
                "GET", "/v2/zones", search_mode="equal", limit=500, name="example.com."
            )
            self.assertEqual(result, "zone123")

    def test_query_zone_id_not_found(self):
        """Test _query_zone_id method when domain is not found"""
        with patch.object(self.provider, "_request") as mock_request:
            mock_request.return_value = {"zones": []}

            result = self.provider._query_zone_id("notfound.com")

            self.assertIsNone(result)

    def test_query_record_success(self):
        """Test _query_record method with successful response"""
        with patch.object(self.provider, "_request") as mock_request:
            mock_request.return_value = {
                "recordsets": [
                    {"id": "rec123", "name": "www.example.com.", "type": "A", "records": ["1.2.3.4"]},
                    {"id": "rec456", "name": "mail.example.com.", "type": "A", "records": ["5.6.7.8"]},
                ]
            }

            result = self.provider._query_record("zone123", "www", "example.com", "A", None, {})

            mock_request.assert_called_once_with(
                "GET",
                "/v2.1/zones/zone123/recordsets",
                limit=500,
                name="www.example.com.",
                type="A",
                line_id=None,
                search_mode="equal",
            )
            self.assertIsNotNone(result)
            if result:  # Type narrowing
                self.assertEqual(result["id"], "rec123")
                self.assertEqual(result["name"], "www.example.com.")

    def test_query_record_with_line(self):
        """Test _query_record method with line parameter"""
        with patch.object(self.provider, "_request") as mock_request:
            mock_request.return_value = {"recordsets": []}

            self.provider._query_record("zone123", "www", "example.com", "A", "line1", {})

            mock_request.assert_called_once_with(
                "GET",
                "/v2.1/zones/zone123/recordsets",
                limit=500,
                name="www.example.com.",
                type="A",
                line_id="line1",
                search_mode="equal",
            )

    def test_query_record_not_found(self):
        """Test _query_record method when no matching record is found"""
        with patch.object(self.provider, "_request") as mock_request:
            mock_request.return_value = {
                "recordsets": [{"id": "rec456", "name": "mail.example.com.", "type": "A", "records": ["5.6.7.8"]}]
            }

            result = self.provider._query_record("zone123", "www", "example.com", "A", None, {})

            self.assertIsNone(result)

    def test_create_record_success(self):
        """Test _create_record method with successful creation"""
        with patch.object(self.provider, "_request") as mock_request:
            mock_request.return_value = {"id": "rec123456"}

            result = self.provider._create_record("zone123", "www", "example.com", "1.2.3.4", "A", 300, "line1", {})

            mock_request.assert_called_once_with(
                "POST",
                "/v2.1/zones/zone123/recordsets",
                name="www.example.com.",
                type="A",
                records=["1.2.3.4"],
                ttl=300,
                line="line1",
                description=self.provider.remark,
            )
            self.assertTrue(result)

    def test_create_record_with_extra_params(self):
        """Test _create_record method with extra parameters"""
        with patch.object(self.provider, "_request") as mock_request:
            mock_request.return_value = {"id": "rec123456"}

            extra = {"description": "Custom description", "tags": ["tag1", "tag2"]}
            result = self.provider._create_record("zone123", "www", "example.com", "1.2.3.4", "A", 300, None, extra)

            mock_request.assert_called_once_with(
                "POST",
                "/v2.1/zones/zone123/recordsets",
                name="www.example.com.",
                type="A",
                records=["1.2.3.4"],
                ttl=300,
                line=None,
                description="Custom description",
                tags=["tag1", "tag2"],
            )
            self.assertTrue(result)

    def test_create_record_failure(self):
        """Test _create_record method with failed creation"""
        provider = HuaweiDNSProvider(self.authid, self.token)

        with patch.object(provider, "_request") as mock_request:
            mock_request.return_value = {"error": "Zone not found"}

            result = provider._create_record("zone123", "www", "example.com", "1.2.3.4", "A", None, None, {})

            self.assertFalse(result)

    def test_update_record_success(self):
        """Test _update_record method with successful update"""
        provider = HuaweiDNSProvider(self.authid, self.token)

        old_record = {"id": "rec123", "name": "www.example.com.", "ttl": 300}

        with patch.object(provider, "_request") as mock_request:
            mock_request.return_value = {"id": "rec123"}

            result = provider._update_record("zone123", old_record, "5.6.7.8", "A", 600, None, {})

            mock_request.assert_called_once_with(
                "PUT",
                "/v2.1/zones/zone123/recordsets/rec123",
                name="www.example.com.",
                type="A",
                records=["5.6.7.8"],
                ttl=600,
                description=provider.remark,
            )
            self.assertTrue(result)

    def test_update_record_with_fallback_ttl(self):
        """Test _update_record method uses old record's TTL when ttl is None"""
        provider = HuaweiDNSProvider(self.authid, self.token)

        old_record = {"id": "rec123", "name": "www.example.com.", "ttl": 300}

        with patch.object(provider, "_request") as mock_request:
            mock_request.return_value = {"id": "rec123"}

            result = provider._update_record("zone123", old_record, "5.6.7.8", "A", None, None, {})

            mock_request.assert_called_once_with(
                "PUT",
                "/v2.1/zones/zone123/recordsets/rec123",
                name="www.example.com.",
                type="A",
                records=["5.6.7.8"],
                ttl=300,
                description=provider.remark,
            )
            self.assertTrue(result)

    def test_update_record_with_extra_params(self):
        """Test _update_record method with extra parameters"""
        provider = HuaweiDNSProvider(self.authid, self.token)

        old_record = {"id": "rec123", "name": "www.example.com.", "ttl": 300}

        with patch.object(provider, "_request") as mock_request:
            mock_request.return_value = {"id": "rec123"}

            extra = {"description": "Updated description", "tags": ["newtag"]}
            result = provider._update_record("zone123", old_record, "5.6.7.8", "A", 600, "line2", extra)

            mock_request.assert_called_once_with(
                "PUT",
                "/v2.1/zones/zone123/recordsets/rec123",
                name="www.example.com.",
                type="A",
                records=["5.6.7.8"],
                ttl=600,
                description="Updated description",
                tags=["newtag"],
            )
            self.assertTrue(result)

    def test_update_record_extra_priority_over_old_record(self):
        """Test that extra parameters take priority over old_record values"""
        provider = HuaweiDNSProvider(self.authid, self.token)

        old_record = {"id": "rec123", "name": "www.example.com.", "ttl": 300, "description": "Old description"}

        with patch.object(provider, "_request") as mock_request:
            mock_request.return_value = {"id": "rec123"}

            # extra should override old_record's description
            extra = {"description": "New description from extra"}
            result = provider._update_record("zone123", old_record, "5.6.7.8", "A", 600, None, extra)

            mock_request.assert_called_once_with(
                "PUT",
                "/v2.1/zones/zone123/recordsets/rec123",
                name="www.example.com.",
                type="A",
                records=["5.6.7.8"],
                ttl=600,
                description="New description from extra",  # Should use extra, not old_record
            )
            self.assertTrue(result)

    def test_update_record_failure(self):
        """Test _update_record method with failed update"""
        provider = HuaweiDNSProvider(self.authid, self.token)

        old_record = {"id": "rec123", "name": "www.example.com."}

        with patch.object(provider, "_request") as mock_request:
            mock_request.return_value = {"error": "Record not found"}

            result = provider._update_record("zone123", old_record, "5.6.7.8", "A", None, None, {})

            self.assertFalse(result)

    def test_line_configuration_support(self):
        """Test that HuaweiDNSProvider supports line configuration"""
        provider = HuaweiDNSProvider(self.authid, self.token)

        with patch.object(provider, "_request") as mock_request:
            mock_request.return_value = {"id": "rec123456"}

            # Test create record with line parameter (line is passed as extra parameter for Huawei)
            result = provider._create_record("zone123", "www", "example.com", "1.2.3.4", "A", 300, "telecom", {})

            # For Huawei DNS, line can be passed as extra parameter
            self.assertTrue(result)
            mock_request.assert_called_once()

    def test_update_record_with_line(self):
        """Test _update_record method with line parameter"""
        provider = HuaweiDNSProvider(self.authid, self.token)

        old_record = {"id": "rec123", "name": "www.example.com."}

        with patch.object(provider, "_request") as mock_request:
            mock_request.return_value = {"id": "rec123"}

            # Test with line parameter (line is handled as needed for different DNS providers)
            result = provider._update_record("zone123", old_record, "5.6.7.8", "A", 600, "unicom", {})

            self.assertTrue(result)
            mock_request.assert_called_once()


class TestHuaweiDNSProviderIntegration(BaseProviderTestCase):
    """Integration test cases for HuaweiDNSProvider - testing with minimal mocking"""

    def setUp(self):
        """Set up test fixtures"""
        super(TestHuaweiDNSProviderIntegration, self).setUp()
        self.authid = "test_access_key"
        self.token = "test_secret_key"

    def test_full_workflow_create_new_record(self):
        """Test complete workflow for creating a new record"""
        provider = HuaweiDNSProvider(self.authid, self.token)

        # Mock only the HTTP layer to simulate API responses
        with patch.object(provider, "_request") as mock_request:
            # Simulate API responses in order: zone query, record query, record creation
            mock_request.side_effect = [
                {"zones": [{"id": "zone123", "name": "example.com."}]},  # _query_zone_id response
                {"recordsets": []},  # _query_record response (no existing record)
                {"id": "rec123456"},  # _create_record response
            ]

            result = provider.set_record("www.example.com", "1.2.3.4", "A", 300, "line1")

            self.assertTrue(result)
            # Verify the actual API calls made
            self.assertEqual(mock_request.call_count, 3)
            mock_request.assert_any_call("GET", "/v2/zones", search_mode="equal", limit=500, name="example.com.")
            mock_request.assert_any_call(
                "GET",
                "/v2.1/zones/zone123/recordsets",
                limit=500,
                name="www.example.com.",
                type="A",
                line_id="line1",
                search_mode="equal",
            )
            mock_request.assert_any_call(
                "POST",
                "/v2.1/zones/zone123/recordsets",
                name="www.example.com.",
                type="A",
                records=["1.2.3.4"],
                ttl=300,
                line="line1",
                description="Managed by [DDNS](https://ddns.newfuture.cc)",
            )

    def test_full_workflow_update_existing_record(self):
        """Test complete workflow for updating an existing record"""
        provider = HuaweiDNSProvider(self.authid, self.token)

        with patch.object(provider, "_request") as mock_request:
            # Simulate API responses
            mock_request.side_effect = [
                {"zones": [{"id": "zone123", "name": "example.com."}]},  # _query_zone_id response
                {  # _query_record response (existing record found)
                    "recordsets": [
                        {"id": "rec123", "name": "www.example.com.", "type": "A", "records": ["5.6.7.8"], "ttl": 300}
                    ]
                },
                {"id": "rec123"},  # _update_record response
            ]

            result = provider.set_record("www.example.com", "1.2.3.4", "A", 300, "line1")

            self.assertTrue(result)
            # Verify the update call was made
            mock_request.assert_any_call(
                "PUT",
                "/v2.1/zones/zone123/recordsets/rec123",
                name="www.example.com.",
                type="A",
                records=["1.2.3.4"],
                ttl=300,
                description="Managed by [DDNS](https://ddns.newfuture.cc)",
            )

    def test_full_workflow_zone_not_found(self):
        """Test complete workflow when zone is not found"""
        provider = HuaweiDNSProvider(self.authid, self.token)

        with patch.object(provider, "_request") as mock_request:
            # Simulate API returning empty zones array
            mock_request.return_value = {"zones": []}

            result = provider.set_record("www.nonexistent.com", "1.2.3.4", "A")
            self.assertFalse(result)

    def test_full_workflow_create_failure(self):
        """Test complete workflow when record creation fails"""
        provider = HuaweiDNSProvider(self.authid, self.token)

        with patch.object(provider, "_request") as mock_request:
            # Simulate responses: zone found, no existing record, creation fails
            mock_request.side_effect = [
                {"zones": [{"id": "zone123", "name": "example.com."}]},  # _query_zone_id response
                {"recordsets": []},  # _query_record response (no existing record)
                {"error": "Zone not found"},  # _create_record fails
            ]

            result = provider.set_record("www.example.com", "1.2.3.4", "A")

            self.assertFalse(result)

    def test_full_workflow_update_failure(self):
        """Test complete workflow when record update fails"""
        provider = HuaweiDNSProvider(self.authid, self.token)

        with patch.object(provider, "_request") as mock_request:
            # Simulate responses: zone found, existing record found, update fails
            mock_request.side_effect = [
                {"zones": [{"id": "zone123", "name": "example.com."}]},  # _query_zone_id response
                {  # _query_record response (existing record found)
                    "recordsets": [
                        {"id": "rec123", "name": "www.example.com.", "type": "A", "records": ["5.6.7.8"], "ttl": 300}
                    ]
                },
                {"error": "Update failed"},  # _update_record fails
            ]

            result = provider.set_record("www.example.com", "1.2.3.4", "A")

            self.assertFalse(result)

    def test_full_workflow_with_extra_options(self):
        """Test complete workflow with additional options"""
        provider = HuaweiDNSProvider(self.authid, self.token)

        with patch.object(provider, "_request") as mock_request:
            # Simulate successful creation with custom options
            mock_request.side_effect = [
                {"zones": [{"id": "zone123", "name": "example.com."}]},  # _query_zone_id response
                {"recordsets": []},  # _query_record response (no existing record)
                {"id": "rec123456"},  # _create_record response
            ]

            result = provider.set_record(
                "www.example.com", "1.2.3.4", "A", 600, "line2", description="Custom record", tags=["production"]
            )

            self.assertTrue(result)
            # Verify that extra parameters are passed through correctly
            mock_request.assert_any_call(
                "POST",
                "/v2.1/zones/zone123/recordsets",
                name="www.example.com.",
                type="A",
                records=["1.2.3.4"],
                ttl=600,
                line="line2",
                description="Custom record",
                tags=["production"],
            )


if __name__ == "__main__":
    unittest.main()



#!/usr/bin/env python
# -*- coding:utf-8 -*-
from re import compile
from os import name as os_name, popen
from socket import socket, getaddrinfo, gethostname, AF_INET, AF_INET6, SOCK_DGRAM
from logging import debug, error

from .util.http import request

# 模块级别的SSL验证配置，默认使用auto模式
ssl_verify = "auto"

# IPV4正则
IPV4_REG = r"((25[0-5]|(2[0-4]|1{0,1}[0-9]){0,1}[0-9])\.){3,3}(25[0-5]|(2[0-4]|1{0,1}[0-9]){0,1}[0-9])"
# IPV6正则
# https://community.helpsystems.com/forums/intermapper/miscellaneous-topics/5acc4fcf-fa83-e511-80cf-0050568460e4
IPV6_REG = r"((([0-9A-Fa-f]{1,4}:){7}([0-9A-Fa-f]{1,4}|:))|(([0-9A-Fa-f]{1,4}:){6}(:[0-9A-Fa-f]{1,4}|((25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)(\.(25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)){3})|:))|(([0-9A-Fa-f]{1,4}:){5}(((:[0-9A-Fa-f]{1,4}){1,2})|:((25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)(\.(25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)){3})|:))|(([0-9A-Fa-f]{1,4}:){4}(((:[0-9A-Fa-f]{1,4}){1,3})|((:[0-9A-Fa-f]{1,4})?:((25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)(\.(25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)){3}))|:))|(([0-9A-Fa-f]{1,4}:){3}(((:[0-9A-Fa-f]{1,4}){1,4})|((:[0-9A-Fa-f]{1,4}){0,2}:((25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)(\.(25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)){3}))|:))|(([0-9A-Fa-f]{1,4}:){2}(((:[0-9A-Fa-f]{1,4}){1,5})|((:[0-9A-Fa-f]{1,4}){0,3}:((25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)(\.(25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)){3}))|:))|(([0-9A-Fa-f]{1,4}:){1}(((:[0-9A-Fa-f]{1,4}){1,6})|((:[0-9A-Fa-f]{1,4}){0,4}:((25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)(\.(25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)){3}))|:))|(:(((:[0-9A-Fa-f]{1,4}){1,7})|((:[0-9A-Fa-f]{1,4}){0,5}:((25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)(\.(25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)){3}))|:)))"  # noqa: E501

# 公网IPv4 API列表，按优先级排序
PUBLIC_IPV4_APIS = [
    "https://api.ipify.cn",
    "https://api.ipify.org",
    "https://4.ipw.cn/",
    "https://ipinfo.io/ip",
    "https://api-ipv4.ip.sb/ip",
    "http://checkip.amazonaws.com",
]

# 公网IPv6 API列表，按优先级排序
PUBLIC_IPV6_APIS = [
    "https://api6.ipify.org/",
    "https://6.ipw.cn/",
    "https://api-ipv6.ip.sb/ip",
    "http://ipv6.icanhazip.com",
]


def default_v4():  # 默认连接外网的ipv4
    s = socket(AF_INET, SOCK_DGRAM)
    s.connect(("1.1.1.1", 53))
    ip = s.getsockname()[0]
    s.close()
    return ip


def default_v6():  # 默认连接外网的ipv6
    s = socket(AF_INET6, SOCK_DGRAM)
    s.connect(("1:1:1:1:1:1:1:1", 8))
    ip = s.getsockname()[0]
    s.close()
    return ip


def local_v6(i=0):  # 本地ipv6地址
    info = getaddrinfo(gethostname(), 0, AF_INET6)
    debug(info)
    return info[int(i)][4][0]


def local_v4(i=0):  # 本地ipv4地址
    info = getaddrinfo(gethostname(), 0, AF_INET)
    debug(info)
    return info[int(i)][-1][0]


def _open(url, reg):
    try:
        debug("open: %s", url)
        # IP 模块重试3次
        response = request("GET", url, verify=ssl_verify, retries=2)
        res = response.body
        debug("response: %s", res)
        match = compile(reg).search(res)
        if match:
            return match.group()
        error("No match found in response: %s", res)
    except Exception as e:
        error(e)


def _try_multiple_apis(api_list, reg, ip_type):
    """
    Try multiple API endpoints until one succeeds
    """
    for url in api_list:
        try:
            debug("Trying %s API: %s", ip_type, url)
            result = _open(url, reg)
            if result:
                debug("Successfully got %s from %s: %s", ip_type, url, result)
                return result
            else:
                debug("No valid IP found from %s", url)
        except Exception as e:
            debug("Failed to get %s from %s: %s", ip_type, url, e)
    return None


def public_v4(url=None, reg=IPV4_REG):  # 公网IPV4地址
    if url:
        # 使用指定URL
        return _open(url, reg)
    else:
        # 使用多个API自动重试
        return _try_multiple_apis(PUBLIC_IPV4_APIS, reg, "IPv4")


def public_v6(url=None, reg=IPV6_REG):  # 公网IPV6地址
    if url:
        # 使用指定URL
        return _open(url, reg)
    else:
        # 使用多个API自动重试
        return _try_multiple_apis(PUBLIC_IPV6_APIS, reg, "IPv6")


def _ip_regex_match(parrent_regex, match_regex):
    ip_pattern = compile(parrent_regex)
    matcher = compile(match_regex)

    if os_name == "nt":  # windows:
        cmd = "ipconfig"
    else:
        cmd = "ip address || ifconfig 2>/dev/null"

    for s in popen(cmd).readlines():
        addr = ip_pattern.search(s)
        if addr and matcher.match(addr.group(1)):
            return addr.group(1)


def regex_v4(reg):  # ipv4 正则提取
    if os_name == "nt":  # Windows: IPv4 xxx: 192.168.1.2
        regex_str = r"IPv4 .*: ((?:\d{1,3}\.){3}\d{1,3})\W"
    else:
        regex_str = r"inet (?:addr\:)?((?:\d{1,3}\.){3}\d{1,3})[\s/]"
    return _ip_regex_match(regex_str, reg)


def regex_v6(reg):  # ipv6 正则提取
    if os_name == "nt":  # Windows: IPv4 xxx: ::1
        regex_str = r"IPv6 .*: ([\:\dabcdef]*)?\W"
    else:
        regex_str = r"inet6 (?:addr\:\s*)?([\:\dabcdef]*)?[\s/%]"
    return _ip_regex_match(regex_str, reg)



# coding=utf-8
"""
Unit tests for AlidnsProvider

@author: Github Copilot
"""

from base_test import BaseProviderTestCase, unittest, patch
from ddns.provider.alidns import AlidnsProvider


class TestAlidnsProvider(BaseProviderTestCase):
    """Test cases for AlidnsProvider"""

    def setUp(self):
        """Set up test fixtures"""
        super(TestAlidnsProvider, self).setUp()
        self.authid = "test_access_key_id"
        self.token = "test_access_key_secret"

    def test_class_constants(self):
        """Test AlidnsProvider class constants"""
        self.assertEqual(AlidnsProvider.endpoint, "https://alidns.aliyuncs.com")
        self.assertEqual(AlidnsProvider.content_type, "application/x-www-form-urlencoded")
        self.assertTrue(AlidnsProvider.decode_response)

    def test_init_with_basic_config(self):
        """Test AlidnsProvider initialization with basic configuration"""
        provider = AlidnsProvider(self.authid, self.token)
        self.assertEqual(provider.id, self.authid)
        self.assertEqual(provider.token, self.token)
        self.assertEqual(provider.endpoint, "https://alidns.aliyuncs.com")

    def test_request_basic(self):
        """Test _request method with basic parameters"""
        provider = AlidnsProvider(self.authid, self.token)

        # Only mock the HTTP call to avoid actual network requests
        with patch.object(provider, "_http") as mock_http:
            mock_http.return_value = {"Success": True}

            result = provider._request("TestAction", DomainName="example.com")

            # Verify HTTP call was made with correct method and path
            mock_http.assert_called_once()
            call_args = mock_http.call_args
            self.assertEqual(call_args[0], ("POST", "/"))

            # Verify body and headers are present
            self.assertIn("body", call_args[1])
            self.assertIn("headers", call_args[1])

            # Verify headers contain required v3 signature fields
            headers = call_args[1]["headers"]
            self.assertIn("Authorization", headers)
            self.assertIn("x-acs-action", headers)
            self.assertIn("x-acs-date", headers)
            self.assertIn("x-acs-version", headers)
            self.assertEqual(headers["x-acs-action"], "TestAction")

            # Verify body is form-encoded
            body = call_args[1]["body"]
            self.assertIn("DomainName=example.com", body)

            self.assertEqual(result, {"Success": True})

    def test_request_filters_none_params(self):
        """Test _request method filters out None parameters"""
        provider = AlidnsProvider(self.authid, self.token)

        # Only mock the HTTP call
        with patch.object(provider, "_http") as mock_http:
            mock_http.return_value = {}

            provider._request("TestAction", DomainName="example.com", TTL=None, Line=None)

            # Verify HTTP call was made
            mock_http.assert_called_once()

            # Body should not contain None parameters
            call_args = mock_http.call_args[1]
            body = call_args.get("body", "")
            self.assertNotIn("TTL=None", body)
            self.assertNotIn("Line=None", body)
            self.assertIn("DomainName=example.com", body)

    def test_split_zone_and_sub_success(self):
        """Test _split_zone_and_sub method with successful response"""
        provider = AlidnsProvider(self.authid, self.token)

        with patch.object(provider, "_http") as mock_http:
            mock_http.return_value = {"DomainName": "example.com", "RR": "sub"}

            main, sub, zone_id = provider._split_zone_and_sub("sub.example.com")

            mock_http.assert_called_once()
            # Verify GetMainDomainName API was called via headers
            call_headers = mock_http.call_args[1]["headers"]
            self.assertEqual(call_headers["x-acs-action"], "GetMainDomainName")

            # Verify the input parameter in body
            call_body = mock_http.call_args[1]["body"]
            self.assertIn("InputString=sub.example.com", call_body)

            self.assertEqual(main, "example.com")
            self.assertEqual(sub, "sub")
            self.assertEqual(zone_id, "example.com")

    def test_split_zone_and_sub_not_found(self):
        """Test _split_zone_and_sub method when domain is not found"""
        provider = AlidnsProvider(self.authid, self.token)

        with patch.object(provider, "_http") as mock_http:
            mock_http.return_value = {}

            main, sub, zone_id = provider._split_zone_and_sub("notfound.com")

            mock_http.assert_called_once()
            self.assertIsNone(main)
            self.assertIsNone(sub)
            self.assertEqual(zone_id, "notfound.com")

    def test_query_record_success_single(self):
        """Test _query_record method with single record found"""
        provider = AlidnsProvider(self.authid, self.token)

        with patch.object(provider, "_http") as mock_http:
            mock_http.return_value = {
                "DomainRecords": {"Record": [{"RR": "www", "RecordId": "123", "Value": "1.2.3.4", "Type": "A"}]}
            }

            result = provider._query_record("example.com", "www", "example.com", "A", None, {})

            mock_http.assert_called_once()
            # Verify DescribeSubDomainRecords API was called via headers
            call_headers = mock_http.call_args[1]["headers"]
            self.assertEqual(call_headers["x-acs-action"], "DescribeSubDomainRecords")

            # Verify parameters in body
            call_body = mock_http.call_args[1]["body"]
            self.assertIn("SubDomain=www.example.com", call_body)
            self.assertIn("DomainName=example.com", call_body)
            self.assertIn("Type=A", call_body)

            self.assertIsNotNone(result)
            if result:  # Type narrowing for mypy
                self.assertEqual(result["RecordId"], "123")
                self.assertEqual(result["RR"], "www")

    def test_query_record_not_found(self):
        """Test _query_record method when no matching record is found"""
        provider = AlidnsProvider(self.authid, self.token)

        with patch.object(provider, "_http") as mock_http:
            mock_http.return_value = {"DomainRecords": {"Record": []}}

            result = provider._query_record("example.com", "www", "example.com", "A", None, {})

            self.assertIsNone(result)

    def test_query_record_empty_response(self):
        """Test _query_record method with empty records response"""
        provider = AlidnsProvider(self.authid, self.token)

        with patch.object(provider, "_http") as mock_http:
            mock_http.return_value = {"DomainRecords": {"Record": []}}

            result = provider._query_record("example.com", "www", "example.com", "A", None, {})

            self.assertIsNone(result)

    def test_query_record_with_extra_params(self):
        """Test _query_record method with extra parameters"""
        provider = AlidnsProvider(self.authid, self.token)

        with patch.object(provider, "_http") as mock_http:
            mock_http.return_value = {"DomainRecords": {"Record": []}}

            extra = {"Lang": "en", "Status": "Enable"}
            provider._query_record("example.com", "www", "example.com", "A", "default", extra)

            mock_http.assert_called_once()
            # Verify extra parameters are included in the request
            call_body = mock_http.call_args[1]["body"]
            self.assertIn("Lang=en", call_body)
            self.assertIn("Status=Enable", call_body)
            self.assertIn("Line=default", call_body)

    def test_create_record_success(self):
        """Test _create_record method with successful creation"""
        provider = AlidnsProvider(self.authid, self.token)

        with patch.object(provider, "_request") as mock_request:
            mock_request.return_value = {"RecordId": "123456"}

            result = provider._create_record("example.com", "www", "example.com", "1.2.3.4", "A", 300, "default", {})

            mock_request.assert_called_once_with(
                "AddDomainRecord",
                DomainName="example.com",
                RR="www",
                Value="1.2.3.4",
                Type="A",
                TTL=300,
                Line="default",
            )
            self.assertTrue(result)

    def test_create_record_failure(self):
        """Test _create_record method with failed creation"""
        provider = AlidnsProvider(self.authid, self.token)

        with patch.object(provider, "_request") as mock_request:
            mock_request.return_value = {"Error": "Invalid domain"}

            result = provider._create_record("example.com", "www", "example.com", "1.2.3.4", "A", None, None, {})

            self.assertFalse(result)

    def test_create_record_with_extra_params(self):
        """Test _create_record method with extra parameters"""
        provider = AlidnsProvider(self.authid, self.token)

        with patch.object(provider, "_request") as mock_request:
            mock_request.return_value = {"RecordId": "123456"}

            extra = {"Priority": 10, "Remark": "Test record"}
            result = provider._create_record("t.com", "www", "t.com", "1.2.3.4", "A", 300, "default", extra)

            mock_request.assert_called_once_with(
                "AddDomainRecord",
                DomainName="t.com",
                RR="www",
                Value="1.2.3.4",
                Type="A",
                TTL=300,
                Line="default",
                Priority=10,
                Remark="Test record",
            )
            self.assertTrue(result)

    def test_update_record_success(self):
        """Test _update_record method with successful update"""
        provider = AlidnsProvider(self.authid, self.token)

        old_record = {"RecordId": "123456", "RR": "www", "Line": "default"}

        with patch.object(provider, "_request") as mock_request:
            mock_request.return_value = {"RecordId": "123456"}

            result = provider._update_record("example.com", old_record, "5.6.7.8", "A", 600, "unicom", {})

            mock_request.assert_called_once_with(
                "UpdateDomainRecord", RecordId="123456", Value="5.6.7.8", RR="www", Type="A", TTL=600, Line="unicom"
            )
            self.assertTrue(result)

    def test_update_record_with_fallback_line(self):
        """Test _update_record method uses old record's line when line is None"""
        provider = AlidnsProvider(self.authid, self.token)

        old_record = {"RecordId": "123456", "RR": "www", "Line": "default"}

        with patch.object(provider, "_request") as mock_request:
            mock_request.return_value = {"RecordId": "123456"}

            result = provider._update_record("example.com", old_record, "5.6.7.8", "A", None, None, {})

            mock_request.assert_called_once_with(
                "UpdateDomainRecord",
                RecordId="123456",
                Value="5.6.7.8",
                RR="www",
                Type="A",
                TTL=None,
                Line="default",  # Should use old record's line
            )
            self.assertTrue(result)

    def test_update_record_failure(self):
        """Test _update_record method with failed update"""
        provider = AlidnsProvider(self.authid, self.token)

        old_record = {"RecordId": "123456", "RR": "www", "Line": "default"}

        with patch.object(provider, "_request") as mock_request:
            mock_request.return_value = {"Error": "Record not found"}

            result = provider._update_record("example.com", old_record, "5.6.7.8", "A", None, None, {})

            self.assertFalse(result)

    def test_update_record_no_changes(self):
        """Test _update_record method when no changes are detected"""
        provider = AlidnsProvider(self.authid, self.token)

        old_record = {"RecordId": "123456", "RR": "www", "Value": "1.2.3.4", "Type": "A", "TTL": 300, "Line": "default"}

        with patch.object(provider, "_request") as mock_request:
            # Same value, type, and TTL should skip update
            result = provider._update_record("example.com", old_record, "1.2.3.4", "A", 300, "default", {})

            # Should return True without making any API calls
            self.assertTrue(result)
            mock_request.assert_not_called()

    def test_update_record_with_extra_params(self):
        """Test _update_record method with extra parameters"""
        provider = AlidnsProvider(self.authid, self.token)

        old_record = {"RecordId": "123456", "RR": "www", "Line": "default"}

        with patch.object(provider, "_request") as mock_request:
            mock_request.return_value = {"RecordId": "123456"}

            extra = {"Priority": 20, "Remark": "Updated record"}
            result = provider._update_record("example.com", old_record, "5.6.7.8", "A", 600, "unicom", extra)

            mock_request.assert_called_once_with(
                "UpdateDomainRecord",
                RecordId="123456",
                Value="5.6.7.8",
                RR="www",
                Type="A",
                TTL=600,
                Line="unicom",
                Priority=20,
                Remark="Updated record",
            )
            self.assertTrue(result)

    def test_update_record_extra_priority_over_old_record(self):
        """Test that extra parameters take priority over old_record values"""
        provider = AlidnsProvider(self.authid, self.token)

        old_record = {"RecordId": "123456", "RR": "www", "Line": "default", "Priority": 10, "Remark": "Old remark"}

        with patch.object(provider, "_request") as mock_request:
            mock_request.return_value = {"RecordId": "123456"}

            # extra should override old_record's Priority and Remark
            extra = {"Priority": 20, "Remark": "New remark from extra"}
            result = provider._update_record("example.com", old_record, "5.6.7.8", "A", 600, "unicom", extra)

            mock_request.assert_called_once_with(
                "UpdateDomainRecord",
                RecordId="123456",
                Value="5.6.7.8",
                RR="www",
                Type="A",
                TTL=600,
                Line="unicom",
                Priority=20,  # Should use extra, not old_record's 10
                Remark="New remark from extra",  # Should use extra, not old_record's "Old remark"
            )
            self.assertTrue(result)

    def test_line_configuration_support(self):
        """Test that AlidnsProvider supports line configuration"""
        provider = AlidnsProvider(self.authid, self.token)

        old_record = {"RecordId": "123456", "RR": "www", "Line": "default"}

        with patch.object(provider, "_request") as mock_request:
            mock_request.return_value = {"RecordId": "123456"}

            # Test with custom line parameter
            result = provider._update_record("example.com", old_record, "5.6.7.8", "A", 600, "telecom", {})

            mock_request.assert_called_once_with(
                "UpdateDomainRecord",
                RecordId="123456",
                Value="5.6.7.8",
                RR="www",
                Type="A",
                TTL=600,
                Line="telecom",  # Should use the provided line parameter
            )
            self.assertTrue(result)

    def test_create_record_with_line(self):
        """Test _create_record method with line parameter"""
        provider = AlidnsProvider(self.authid, self.token)

        with patch.object(provider, "_request") as mock_request:
            mock_request.return_value = {"RecordId": "123456"}

            result = provider._create_record("example.com", "www", "example.com", "1.2.3.4", "A", 300, "unicom", {})

            mock_request.assert_called_once_with(
                "AddDomainRecord", DomainName="example.com", RR="www", Value="1.2.3.4", Type="A", TTL=300, Line="unicom"
            )
            self.assertTrue(result)


class TestAlidnsProviderIntegration(BaseProviderTestCase):
    """Integration test cases for AlidnsProvider - testing with minimal mocking"""

    def setUp(self):
        """Set up test fixtures"""
        super(TestAlidnsProviderIntegration, self).setUp()
        self.authid = "test_access_key_id"
        self.token = "test_access_key_secret"

    def test_full_workflow_create_new_record(self):
        """Test complete workflow for creating a new record"""
        provider = AlidnsProvider(self.authid, self.token)

        # Mock only the HTTP layer to simulate API responses
        with patch.object(provider, "_http") as mock_http:
            # Simulate API responses in order: GetMainDomainName, DescribeSubDomainRecords, AddDomainRecord
            mock_http.side_effect = [
                {"DomainName": "example.com", "RR": "www"},  # _split_zone_and_sub response
                {"DomainRecords": {"Record": []}},  # _query_record response (no existing record)
                {"RecordId": "123456"},  # _create_record response
            ]

            result = provider.set_record("www.example.com", "1.2.3.4", "A", 300, "default")

            self.assertTrue(result)
            # Verify the actual HTTP calls made - should be 3 calls
            self.assertEqual(mock_http.call_count, 3)

            # Check that proper API actions were called by examining request headers
            call_actions = [call[1]["headers"]["x-acs-action"] for call in mock_http.call_args_list]
            self.assertIn("GetMainDomainName", call_actions)
            self.assertIn("DescribeSubDomainRecords", call_actions)
            self.assertIn("AddDomainRecord", call_actions)

    def test_full_workflow_update_existing_record(self):
        """Test complete workflow for updating an existing record"""
        provider = AlidnsProvider(self.authid, self.token)

        with patch.object(provider, "_http") as mock_http:
            # Simulate API responses: GetMainDomainName, DescribeSubDomainRecords, UpdateDomainRecord
            mock_http.side_effect = [
                {"DomainName": "example.com", "RR": "www"},  # _split_zone_and_sub response
                {  # _query_record response (existing record found)
                    "DomainRecords": {
                        "Record": [
                            {"RecordId": "123456", "RR": "www", "Value": "5.6.7.8", "Type": "A", "Line": "default"}
                        ]
                    }
                },
                {"RecordId": "123456"},  # _update_record response
            ]

            result = provider.set_record("www.example.com", "1.2.3.4", "A", 300, "default")

            self.assertTrue(result)
            # Verify 3 HTTP calls were made
            self.assertEqual(mock_http.call_count, 3)

            # Check that UpdateDomainRecord was called
            call_actions = [call[1]["headers"]["x-acs-action"] for call in mock_http.call_args_list]
            self.assertIn("UpdateDomainRecord", call_actions)

    def test_full_workflow_zone_not_found(self):
        """Test complete workflow when zone is not found"""
        provider = AlidnsProvider(self.authid, self.token)

        with patch.object(provider, "_http") as mock_http:
            # Simulate API returning empty response for zone query
            mock_http.return_value = {}

            # Should return False when zone not found
            result = provider.set_record("www.nonexistent.com", "1.2.3.4", "A")
            self.assertFalse(result)

    def test_full_workflow_create_failure(self):
        """Test complete workflow when record creation fails"""
        provider = AlidnsProvider(self.authid, self.token)

        with patch.object(provider, "_http") as mock_http:
            # Simulate responses: zone found, no existing record, creation fails
            mock_http.side_effect = [
                {"DomainName": "example.com", "RR": "www"},  # _split_zone_and_sub response
                {"DomainRecords": {"Record": []}},  # _query_record response (no existing record)
                {"Error": "API error", "Code": "InvalidParameter"},  # _create_record fails
            ]

            result = provider.set_record("www.example.com", "1.2.3.4", "A")

            self.assertFalse(result)

    def test_full_workflow_update_failure(self):
        """Test complete workflow when record update fails"""
        provider = AlidnsProvider(self.authid, self.token)

        with patch.object(provider, "_http") as mock_http:
            # Simulate responses: zone found, existing record found, update fails
            mock_http.side_effect = [
                {"DomainName": "example.com", "RR": "www"},  # _split_zone_and_sub response
                {  # _query_record response (existing record found)
                    "DomainRecords": {
                        "Record": [
                            {"RecordId": "123456", "RR": "www", "Value": "5.6.7.8", "Type": "A", "Line": "default"}
                        ]
                    }
                },
                {"Error": "API error", "Code": "InvalidParameter"},  # _update_record fails
            ]

            result = provider.set_record("www.example.com", "1.2.3.4", "A")

            self.assertFalse(result)

    def test_full_workflow_with_options(self):
        """Test complete workflow with additional options like ttl and line"""
        provider = AlidnsProvider(self.authid, self.token)

        with patch.object(provider, "_http") as mock_http:
            # Simulate successful creation with custom options
            mock_http.side_effect = [
                {"DomainName": "example.com", "RR": "www"},  # _split_zone_and_sub response
                {"DomainRecords": {"Record": []}},  # _query_record response (no existing record)
                {"RecordId": "123456"},  # _create_record response
            ]

            result = provider.set_record("www.example.com", "1.2.3.4", "A", 600, "unicom")

            self.assertTrue(result)
            # Verify that extra parameters are passed through correctly
            self.assertEqual(mock_http.call_count, 3)

            # Check that the create call contains the correct parameters
            # Find the AddDomainRecord call (should be the last one)
            add_record_call = None
            for call in mock_http.call_args_list:
                if call[1]["headers"]["x-acs-action"] == "AddDomainRecord":
                    add_record_call = call
                    break

            self.assertIsNotNone(add_record_call)
            if add_record_call:
                create_body = add_record_call[1]["body"]
                self.assertIn("TTL=600", create_body)
                self.assertIn("Line=unicom", create_body)


if __name__ == "__main__":
    unittest.main()



# -*- coding: utf-8 -*-
# type: ignore[index,operator,assignment]
"""
Test cases for cache module

@author: GitHub Copilot
"""

from __init__ import patch, unittest

import os
import tempfile
from time import sleep
from ddns.cache import Cache  # noqa: E402


class TestCache(unittest.TestCase):
    """Test cases for Cache class"""

    def setUp(self):
        """Set up test fixtures"""
        # Create a temporary directory for test cache files
        self.cache_file = tempfile.mktemp(prefix="ddns_test_cache_", suffix="pk1")
        # self.cache_file = os.path.join(self.test_dir, "test_cache.%d.pkl" % random.randint(1, 10000))

    def tearDown(self):
        """Clean up test fixtures"""
        # Remove the temporary directory and all its contents
        if os.path.exists(self.cache_file):
            os.remove(self.cache_file)

    def test_init_new_cache(self):
        """Test cache initialization with new cache file"""
        cache = Cache(self.cache_file)

        # Verify initialization
        self.assertEqual(len(cache), 0)
        self.assertIsInstance(cache.time, float)
        self.assertFalse(os.path.exists(self.cache_file))  # File not created until sync

    def test_init_with_logger(self):
        """Test cache initialization with custom logger"""
        import logging

        logger = logging.getLogger("test_logger")
        cache = Cache(self.cache_file, logger=logger)

        self.assertEqual(len(cache), 0)

    def test_init_with_sync(self):
        """Test cache initialization with sync enabled"""
        cache = Cache(self.cache_file, sync=True)

        self.assertEqual(len(cache), 0)

    def test_setitem_and_getitem(self):
        """Test setting and getting cache items"""
        cache = Cache(self.cache_file)

        # Test setting items
        cache["key1"] = "value1"
        cache["key2"] = "value2"

        # Test getting items
        self.assertEqual(cache["key1"], "value1")
        self.assertEqual(cache["key2"], "value2")
        self.assertEqual(len(cache), 2)

    def test_setitem_duplicate_value(self):
        """Test setting the same value twice doesn't trigger update"""
        cache = Cache(self.cache_file)

        with patch.object(cache, "_Cache__update") as mock_update:
            cache["key1"] = "value1"
            mock_update.assert_called_once()

            # Setting the same value should not trigger update
            mock_update.reset_mock()
            cache["key1"] = "value1"
            mock_update.assert_not_called()

    def test_delitem(self):
        """Test deleting cache items"""
        cache = Cache(self.cache_file)
        cache["key1"] = "value1"
        cache["key2"] = "value2"

        # Delete an item
        del cache["key1"]

        self.assertEqual(len(cache), 1)
        self.assertNotIn("key1", cache)
        self.assertIn("key2", cache)

    def test_delitem_nonexistent_key(self):
        """Test deleting non-existent key doesn't raise error (silent handling)"""
        cache = Cache(self.cache_file)
        cache["key1"] = "value1"

        # Should not raise any exception
        del cache["nonexistent"]

        # Original data should remain unchanged
        self.assertEqual(len(cache), 1)
        self.assertIn("key1", cache)

    def test_delitem_idempotent(self):
        """Test that multiple deletions of the same key are safe"""
        cache = Cache(self.cache_file)
        cache["key1"] = "value1"

        # First deletion should work
        del cache["key1"]
        self.assertEqual(len(cache), 0)
        self.assertNotIn("key1", cache)

        # Second deletion should be safe (no error)
        del cache["key1"]
        self.assertEqual(len(cache), 0)

        # Third deletion should also be safe
        del cache["key1"]
        self.assertEqual(len(cache), 0)

    def test_contains(self):
        """Test membership testing"""
        cache = Cache(self.cache_file)
        cache["key1"] = "value1"

        self.assertIn("key1", cache)
        self.assertNotIn("key2", cache)

    def test_clear(self):
        """Test clearing cache"""
        cache = Cache(self.cache_file)
        cache["key1"] = "value1"
        cache["key2"] = "value2"

        cache.clear()

        self.assertEqual(len(cache), 0)
        self.assertNotIn("key1", cache)
        self.assertNotIn("key2", cache)

    def test_clear_empty_cache(self):
        """Test clearing empty cache doesn't trigger update"""
        cache = Cache(self.cache_file)

        with patch.object(cache, "_Cache__update") as mock_update:
            cache.clear()
            mock_update.assert_not_called()

    def test_clear_preserves_private_fields(self):
        """Test that clear only removes non-private fields"""
        cache = Cache(self.cache_file)
        cache["normal1"] = "value1"
        cache["normal2"] = "value2"
        cache["__private"] = "private_value"

        # Check initial state
        self.assertEqual(len(cache), 2)  # Only counts non-private fields

        # Clear should only remove non-private fields
        cache.clear()

        # Private field should still exist in underlying dict
        self.assertEqual(len(cache), 0)
        self.assertNotIn("normal1", cache)
        self.assertNotIn("normal2", cache)
        # Private field still exists but not counted/visible
        self.assertTrue("__private" in dict(cache))

    def test_iteration(self):
        """Test iterating over cache keys"""
        cache = Cache(self.cache_file)
        cache["key1"] = "value1"
        cache["key2"] = "value2"

        keys = list(cache)
        self.assertEqual(set(keys), {"key1", "key2"})

    def test_private_fields_excluded(self):
        """Test that private fields (starting with __) are excluded from operations"""
        cache = Cache(self.cache_file)
        cache["normal_key"] = "normal_value"

        # Manually add a private field to the underlying dict (for testing purposes)
        super(Cache, cache).__setitem__("__private_field", "private_value")

        # len() should exclude private fields
        self.assertEqual(len(cache), 1)

        # iteration should exclude private fields
        keys = list(cache)
        self.assertEqual(keys, ["normal_key"])

        # data() should exclude private fields
        data = cache.get(None)
        self.assertEqual(data, {"normal_key": "normal_value"})

    def test_private_field_operations_no_sync(self):
        """Test that private field operations don't trigger sync"""
        cache = Cache(self.cache_file)

        with patch.object(cache, "_Cache__update") as mock_update:
            # Setting private field should not trigger sync
            cache["__private"] = "private_value"
            mock_update.assert_not_called()

            # Modifying private field should not trigger sync
            cache["__private"] = "new_private_value"
            mock_update.assert_not_called()

            # Deleting private field should not trigger sync
            del cache["__private"]
            mock_update.assert_not_called()

            # Normal field operations should trigger sync
            cache["normal"] = "value"
            mock_update.assert_called_once()

    def test_private_fields_not_saved_to_file(self):
        """Test that private fields are not saved to file"""
        cache = Cache(self.cache_file)
        cache["normal_key"] = "normal_value"
        cache["__private_key"] = "private_value"

        # Sync to file
        cache.sync()

        # Load new cache instance
        cache2 = Cache(self.cache_file)

        # Only normal fields should be loaded
        self.assertEqual(len(cache2), 1)
        self.assertIn("normal_key", cache2)
        self.assertNotIn("__private_key", cache2)
        self.assertEqual(cache2["normal_key"], "normal_value")

    def test_data_method(self):
        """Test data method for getting cache contents"""
        cache = Cache(self.cache_file)
        cache["key1"] = "value1"
        cache["key2"] = "value2"

        # Test getting all data
        # data = cache.get()
        # self.assertEqual(data, {"key1": "value1", "key2": "value2"})

        # Test getting specific key
        self.assertEqual(cache.get("key1"), "value1")
        self.assertEqual(cache.get("nonexistent", "default"), "default")

    def test_data_method_with_sync(self):
        """Test data method with sync enabled calls load"""
        cache = Cache(self.cache_file, sync=True)

        with patch.object(cache, "load") as mock_load:
            cache.load()

            mock_load.assert_called_once()

    def test_sync_method(self):
        """Test sync method saves data to file"""
        cache = Cache(self.cache_file)
        cache["key1"] = "value1"
        cache["key2"] = "value2"

        # Sync should save to file
        result = cache.sync()

        self.assertIs(result, cache)  # Should return self
        self.assertTrue(os.path.exists(self.cache_file))

        # Load another cache instance to verify data was saved
        cache2 = Cache(self.cache_file)
        self.assertEqual(len(cache2), 2)
        self.assertEqual(cache2["key1"], "value1")
        self.assertEqual(cache2["key2"], "value2")

    def test_sync_no_changes(self):
        """Test sync when no changes have been made after load"""
        # Create and save initial cache
        cache = Cache(self.cache_file)
        cache["key1"] = "value1"
        cache.sync()  # This clears the __changed flag

        with patch("ddns.cache.dump") as mock_dump:
            cache.sync()  # This should not call dump since no changes
            mock_dump.assert_not_called()

    def test_load_existing_file(self):
        """Test loading from existing cache file"""
        # Create initial cache and save data
        cache1 = Cache(self.cache_file)
        cache1["key1"] = "value1"
        cache1["key2"] = 2
        cache1.sync()

        # Load new cache instance
        cache2 = Cache(self.cache_file)

        self.assertEqual(len(cache2), 2)
        self.assertEqual(cache2["key1"], "value1")
        self.assertEqual(cache2["key2"], 2)

    def test_load_corrupted_file(self):
        """Test loading from corrupted cache file"""
        # Create a corrupted cache file
        with open(self.cache_file, "w") as f:
            f.write("corrupted data")

        # Should handle corruption gracefully
        cache = Cache(self.cache_file)
        self.assertEqual(len(cache), 0)

    def test_load_with_exception(self):
        """Test load method handles exceptions properly"""
        # Create a file first
        with open(self.cache_file, "wb") as f:
            f.write(b"invalid pickle data")

        cache = Cache(self.cache_file)

        with patch("ddns.cache.load", side_effect=Exception("Test error")):
            with patch.object(cache, "_Cache__logger") as mock_logger:
                cache.load()
                mock_logger.warning.assert_called_once()

    def test_time_property(self):
        """Test time property returns modification time"""
        cache = Cache(self.cache_file)
        initial_time = cache.time  # type: float # type: ignore[assignment]

        self.assertIsInstance(initial_time, float)
        self.assertGreater(initial_time, 0)

    def test_close_method(self):
        """Test close method syncs and cleans up"""
        cache = Cache(self.cache_file)
        cache["key1"] = "value1"

        with patch.object(cache, "sync") as mock_sync:
            cache.close()
            mock_sync.assert_called_once()

    def test_str_representation(self):
        """Test string representation of cache"""
        cache = Cache(self.cache_file)
        cache["key1"] = "value1"

        str_repr = str(cache)
        self.assertIn("key1", str_repr)
        self.assertIn("value1", str_repr)

    def test_auto_sync_behavior(self):
        """Test auto-sync behavior when sync=True"""
        cache = Cache(self.cache_file, sync=True)

        with patch.object(cache, "sync") as mock_sync:
            cache["key1"] = "value1"
            mock_sync.assert_called()

    def test_no_auto_sync_behavior(self):
        """Test no auto-sync behavior when sync=False (default)"""
        cache = Cache(self.cache_file, sync=False)

        with patch.object(cache, "sync") as mock_sync:
            cache["key1"] = "value1"
            mock_sync.assert_not_called()

    def test_context_manager_like_usage(self):
        """Test using cache in a context-manager-like pattern"""
        cache = Cache(self.cache_file)
        cache["key1"] = "value1"
        cache["key2"] = "value2"

        # Manually call close (simulating __del__)
        cache.close()

        # Verify data was persisted
        cache2 = Cache(self.cache_file)
        self.assertEqual(len(cache2), 2)
        self.assertEqual(cache2["key1"], "value1")

    def test_update_time_on_changes(self):
        """Test that modification time is updated on changes"""
        cache = Cache(self.cache_file)
        initial_time = cache.time

        # Small delay to ensure time difference
        sleep(0.01)

        cache["key1"] = "value1"
        new_time = cache.time  # type: float # type: ignore[assignment]

        self.assertGreater(new_time, initial_time)  # type: ignore[comparison-overlap]

    def test_integration_multiple_operations(self):
        """Integration test with multiple operations"""
        cache = Cache(self.cache_file)

        # Add some data
        cache["user1"] = {"name": "Alice", "age": 30}
        cache["user2"] = {"name": "Bob", "age": 25}
        cache["config"] = {"debug": True, "timeout": 30}

        self.assertEqual(len(cache), 3)

        # Modify data
        cache["user1"]["age"] = 31  # This won't trigger update automatically
        cache["user1"] = {"name": "Alice", "age": 31}  # This will

        # Delete data
        del cache["user2"]

        self.assertEqual(len(cache), 2)
        self.assertEqual(cache["user1"]["age"], 31)
        self.assertNotIn("user2", cache)

        # Persist and reload
        cache.sync()

        cache2 = Cache(self.cache_file)
        self.assertEqual(len(cache2), 2)
        self.assertEqual(cache2["user1"]["age"], 31)
        self.assertEqual(cache2["config"]["debug"], True)

    def test_mixed_public_private_operations(self):
        """Test mixed operations with public and private fields"""
        cache = Cache(self.cache_file)

        # Add mixed data
        cache["public1"] = "public_value1"
        cache["__private1"] = "private_value1"
        cache["public2"] = "public_value2"
        cache["__private2"] = "private_value2"

        # Only public fields should be counted
        self.assertEqual(len(cache), 2)

        # Only public fields should be iterable
        public_keys = list(cache)
        self.assertEqual(set(public_keys), {"public1", "public2"})

        # data() should only return public fields
        data = cache.get(None)
        self.assertEqual(data, {"public1": "public_value1", "public2": "public_value2"})

        # Delete operations
        del cache["public1"]  # Should work
        del cache["__private1"]  # Should work but not trigger sync
        del cache["nonexistent"]  # Should be safe

        # Only one public field should remain
        self.assertEqual(len(cache), 1)
        self.assertEqual(list(cache), ["public2"])

        # Sync and reload
        cache.sync()
        cache2 = Cache(self.cache_file)

        # Only public field should be persisted
        self.assertEqual(len(cache2), 1)
        self.assertEqual(list(cache2), ["public2"])
        self.assertEqual(cache2["public2"], "public_value2")

    def test_str_representation_excludes_private(self):
        """Test that string representation only shows public fields"""
        cache = Cache(self.cache_file)
        cache["public"] = "public_value"
        cache["__private"] = "private_value"

        str_repr = str(cache)
        self.assertIn("public", str_repr)
        self.assertIn("public_value", str_repr)
        # Note: private fields might still appear in str() since it calls super().__str__()
        # This is acceptable as str() shows the raw dict content

    def test_json_format_verification(self):
        """Test that cache files are saved in JSON format"""
        import json

        cache = Cache(self.cache_file)
        cache["string_key"] = "string_value"
        cache["number_key"] = 42
        cache["list_key"] = [1, 2, 3]
        cache["dict_key"] = {"nested": "value"}

        # Save to file
        cache.sync()

        # Verify file exists and is valid JSON
        self.assertTrue(os.path.exists(self.cache_file))

        # Read the file and verify it's valid JSON
        with open(self.cache_file, "r") as f:
            file_content = f.read()
            parsed_json = json.loads(file_content)

        # Verify the content matches what we saved
        self.assertEqual(parsed_json["string_key"], "string_value")
        self.assertEqual(parsed_json["number_key"], 42)
        self.assertEqual(parsed_json["list_key"], [1, 2, 3])
        self.assertEqual(parsed_json["dict_key"], {"nested": "value"})

        # Verify the file contains readable JSON (not binary pickle data)
        self.assertIn('"string_key"', file_content)
        self.assertIn('"string_value"', file_content)

    def test_cache_new_disabled(self):
        """Test Cache.new with cache disabled (config_cache=False)"""
        import logging

        logger = logging.getLogger("test_logger")
        cache = Cache.new(False, "test_hash", logger)

        # Should return None when cache is disabled
        self.assertIsNone(cache)

    def test_cache_new_temp_file(self):
        """Test Cache.new with temp file creation (config_cache=True)"""
        import logging
        import tempfile

        logger = logging.getLogger("test_logger")
        test_hash = "test_hash_123"
        cache = Cache.new(True, test_hash, logger)

        # Should create a cache instance
        self.assertIsNotNone(cache)
        self.assertIsInstance(cache, Cache)

        # Should use temp directory with correct naming pattern
        expected_pattern = "ddns.%s.cache" % test_hash
        self.assertIn(expected_pattern, cache._Cache__filename)
        self.assertIn(tempfile.gettempdir(), cache._Cache__filename)

        # Clean up
        cache.close()

    def test_cache_new_custom_path(self):
        """Test Cache.new with custom cache file path"""
        import logging

        logger = logging.getLogger("test_logger")
        custom_path = self.cache_file
        cache = Cache.new(custom_path, "unused_hash", logger)

        # Should create a cache instance with custom path
        self.assertIsNotNone(cache)
        self.assertIsInstance(cache, Cache)
        self.assertEqual(cache._Cache__filename, custom_path)

        # Clean up
        cache.close()

    @patch("ddns.cache.time")
    def test_cache_new_outdated_cache(self, mock_time):
        """Test Cache.new with outdated cache file (>72 hours old)"""
        import logging
        import json

        logger = logging.getLogger("test_logger")

        # Create a cache file with some data
        test_data = {"test_key": "test_value"}
        with open(self.cache_file, "w") as f:
            json.dump(test_data, f)

        # Mock current time to make cache appear outdated
        # Cache file mtime will be recent, but we'll mock current time to be 73 hours later
        current_time = 1000000
        mock_time.return_value = current_time

        # Mock the file modification time to be 73 hours ago
        old_mtime = current_time - (73 * 3600)  # 73 hours ago

        with patch("ddns.cache.stat") as mock_stat:
            mock_stat.return_value.st_mtime = old_mtime
            cache = Cache.new(self.cache_file, "test_hash", logger)

        # Should create cache instance but clear outdated data
        self.assertIsNotNone(cache)
        self.assertIsInstance(cache, Cache)
        self.assertEqual(len(cache), 0)  # Should be empty due to age

        # Clean up
        cache.close()

    def test_cache_new_empty_cache(self):
        """Test Cache.new with empty cache file"""
        import logging
        import json

        logger = logging.getLogger("test_logger")

        # Create an empty cache file
        with open(self.cache_file, "w") as f:
            json.dump({}, f)

        cache = Cache.new(self.cache_file, "test_hash", logger)

        # Should create cache instance
        self.assertIsNotNone(cache)
        self.assertIsInstance(cache, Cache)
        self.assertEqual(len(cache), 0)

        # Clean up
        cache.close()

    @patch("ddns.cache.time")
    def test_cache_new_valid_cache(self, mock_time):
        """Test Cache.new with valid cache file with data"""
        import logging
        import json

        logger = logging.getLogger("test_logger")

        # Create a cache file with test data
        test_data = {
            "domain1.com": {"ip": "1.2.3.4", "timestamp": 1234567890},
            "domain2.com": {"ip": "5.6.7.8", "timestamp": 1234567891},
        }
        with open(self.cache_file, "w") as f:
            json.dump(test_data, f)

        # Mock current time to be within 72 hours of cache creation
        current_time = 1000000
        mock_time.return_value = current_time

        # Mock file modification time to be recent (within 72 hours)
        recent_mtime = current_time - (24 * 3600)  # 24 hours ago

        with patch("ddns.cache.stat") as mock_stat:
            mock_stat.return_value.st_mtime = recent_mtime
            cache = Cache.new(self.cache_file, "test_hash", logger)

        # Should create cache instance with loaded data
        self.assertIsNotNone(cache)
        self.assertIsInstance(cache, Cache)
        self.assertEqual(len(cache), 2)
        self.assertEqual(cache["domain1.com"]["ip"], "1.2.3.4")
        self.assertEqual(cache["domain2.com"]["ip"], "5.6.7.8")

        # Clean up
        cache.close()

    def test_cache_new_nonexistent_file(self):
        """Test Cache.new with nonexistent cache file"""
        import logging

        logger = logging.getLogger("test_logger")
        nonexistent_path = tempfile.mktemp(prefix="ddns_test_nonexistent_", suffix=".cache")

        # Ensure file doesn't exist
        if os.path.exists(nonexistent_path):
            os.remove(nonexistent_path)

        cache = Cache.new(nonexistent_path, "test_hash", logger)

        # Should create cache instance with empty data
        self.assertIsNotNone(cache)
        self.assertIsInstance(cache, Cache)
        self.assertEqual(len(cache), 0)

        # Clean up
        cache.close()


if __name__ == "__main__":
    unittest.main()



# coding=utf-8
"""
AliDNS API
阿里DNS解析操作库
@author: NewFuture
"""

from time import gmtime, strftime, time

from ._base import TYPE_FORM, BaseProvider, encode_params, join_domain
from ._signature import hmac_sha256_authorization, sha256_hash


class AliBaseProvider(BaseProvider):
    """阿里云基础Provider，提供通用的_request方法"""

    endpoint = "https://alidns.aliyuncs.com"
    content_type = TYPE_FORM  # 阿里云DNS API使用表单格式
    api_version = "2015-01-09"  # API版本，v3签名需要

    def _request(self, action, method="POST", **params):
        # type: (str, str, **(Any)) -> dict
        """Aliyun v3 https://help.aliyun.com/zh/sdk/product-overview/v3-request-structure-and-signature"""
        params = {k: v for k, v in params.items() if v is not None}

        if method in ("GET", "DELETE"):
            # For GET and DELETE requests, parameters go in query string
            query_string = encode_params(params) if len(params) > 0 else ""
            body_content = ""
        else:
            # For POST requests, parameters go in body
            body_content = self._encode_body(params)
            query_string = ""

        path = "/"
        content_hash = sha256_hash(body_content)
        # 构造请求头部
        headers = {
            "host": self.endpoint.split("://", 1)[1].strip("/"),
            "content-type": self.content_type,
            "x-acs-action": action,
            "x-acs-content-sha256": content_hash,
            "x-acs-date": strftime("%Y-%m-%dT%H:%M:%SZ", gmtime()),
            "x-acs-signature-nonce": str(hash(time()))[2:],
            "x-acs-version": self.api_version,
        }

        # 使用通用签名函数
        authorization = hmac_sha256_authorization(
            secret_key=self.token,
            method=method,
            path=path,
            query=query_string,
            headers=headers,
            body_hash=content_hash,
            signing_string_format="ACS3-HMAC-SHA256\n{HashedCanonicalRequest}",
            authorization_format=(
                "ACS3-HMAC-SHA256 Credential=" + self.id + ",SignedHeaders={SignedHeaders},Signature={Signature}"
            ),
        )
        headers["Authorization"] = authorization
        # 对于v3签名的RPC API，参数在request body中
        path = path if not query_string else path + "?" + format(query_string)
        return self._http(method, path, body=body_content, headers=headers)


class AlidnsProvider(AliBaseProvider):
    """阿里云DNS Provider"""

    def _split_zone_and_sub(self, domain):
        # type: (str) -> tuple[str | None, str | None, str]
        """
        AliDNS 支持直接查询主域名和RR，无需循环查询。
        返回没有DomainId,用DomainName代替
        https://help.aliyun.com/zh/dns/api-alidns-2015-01-09-getmaindomainname
        """
        res = self._request("GetMainDomainName", InputString=domain)
        sub, main = res.get("RR"), res.get("DomainName")
        return (main, sub, main or domain)

    def _query_zone_id(self, domain):
        """调用_split_zone_and_sub可直接获取，无需调用_query_zone_id"""
        raise NotImplementedError("_split_zone_and_sub is used to get zone_id")

    def _query_record(self, zone_id, subdomain, main_domain, record_type, line, extra):
        """https://help.aliyun.com/zh/dns/api-alidns-2015-01-09-describesubdomainrecords"""
        sub = join_domain(subdomain, main_domain)
        data = self._request(
            "DescribeSubDomainRecords",
            SubDomain=sub,  # aliyun API要求SubDomain为完整域名
            DomainName=main_domain,
            Type=record_type,
            Line=line,
            PageSize=500,
            Lang=extra.get("Lang"),  # 默认中文
            Status=extra.get("Status"),  # 默认全部状态
        )
        records = data.get("DomainRecords", {}).get("Record", [])
        if not records:
            self.logger.warning(
                "No records found for [%s] with %s <%s> (line: %s)", zone_id, subdomain, record_type, line
            )
        elif not isinstance(records, list):
            self.logger.error("Invalid records format: %s", records)
        else:
            return next((r for r in records), None)
        return None

    def _create_record(self, zone_id, subdomain, main_domain, value, record_type, ttl, line, extra):
        """https://help.aliyun.com/zh/dns/api-alidns-2015-01-09-adddomainrecord"""
        data = self._request(
            "AddDomainRecord",
            DomainName=main_domain,
            RR=subdomain,
            Value=value,
            Type=record_type,
            TTL=ttl,
            Line=line,
            **extra
        )  # fmt: skip
        if data and data.get("RecordId"):
            self.logger.info("Record created: %s", data)
            return True
        self.logger.error("Failed to create record: %s", data)
        return False

    def _update_record(self, zone_id, old_record, value, record_type, ttl, line, extra):
        """https://help.aliyun.com/zh/dns/api-alidns-2015-01-09-updatedomainrecord"""
        # 阿里云DNS update新旧值不能一样，先判断是否发生变化
        if (
            old_record.get("Value") == value
            and old_record.get("Type") == record_type
            and (not ttl or old_record.get("TTL") == ttl)
        ):
            domain = join_domain(old_record.get("RR"), old_record.get("DomainName"))
            self.logger.warning("No changes detected, skipping update for record: %s", domain)
            return True
        data = self._request(
            "UpdateDomainRecord",
            RecordId=old_record.get("RecordId"),
            Value=value,
            RR=old_record.get("RR"),
            Type=record_type,
            TTL=ttl,
            Line=line or old_record.get("Line"),
            **extra
        )  # fmt: skip
        if data and data.get("RecordId"):
            self.logger.info("Record updated: %s", data)
            return True
        self.logger.error("Failed to update record: %s", data)
        return False



# -*- coding:utf-8 -*-
"""
Systemd timer-based task scheduler for Linux
@author: NewFuture
"""

import os
import re

from ..util.fileio import read_file_safely, write_file
from ..util.try_run import try_run
from ._base import BaseScheduler

try:  # python 3
    PermissionError  # type: ignore
except NameError:  # python 2 doesn't have PermissionError, use OSError instead
    PermissionError = IOError


class SystemdScheduler(BaseScheduler):
    """Systemd timer-based task scheduler for Linux"""

    SERVICE_NAME = "ddns.service"
    TIMER_NAME = "ddns.timer"
    SERVICE_PATH = "/etc/systemd/system/ddns.service"
    TIMER_PATH = "/etc/systemd/system/ddns.timer"

    def _systemctl(self, *args):
        """Run systemctl command and return success status"""
        result = try_run(["systemctl"] + list(args), logger=self.logger)
        return result is not None

    def is_installed(self):
        """Check if systemd timer files exist"""
        return os.path.exists(self.SERVICE_PATH) and os.path.exists(self.TIMER_PATH)

    def get_status(self):
        """Get comprehensive status information"""
        installed = self.is_installed()
        status = {"scheduler": "systemd", "installed": installed}
        if not installed:
            return status

        # Check if timer is enabled
        result = try_run(["systemctl", "is-enabled", self.TIMER_NAME], logger=self.logger)
        status["enabled"] = bool(result and result.strip() == "enabled")

        # Extract interval from timer file
        timer_content = read_file_safely(self.TIMER_PATH) or ""
        match = re.search(r"OnUnitActiveSec=(\d+)m", timer_content)
        status["interval"] = int(match.group(1)) if match else None

        # Extract command and description from service file
        service_content = read_file_safely(self.SERVICE_PATH) or ""
        match = re.search(r"ExecStart=(.+)", service_content)
        status["command"] = match.group(1).strip() if match else None
        desc_match = re.search(r"Description=(.+)", service_content)
        status["description"] = desc_match.group(1).strip() if desc_match else None

        return status

    def install(self, interval, ddns_args=None):
        """Install systemd timer with specified interval"""
        ddns_commands = self._build_ddns_command(ddns_args)
        # Convert array to properly quoted command string for ExecStart
        ddns_command = self._quote_command_array(ddns_commands)
        work_dir = os.getcwd()
        description = self._get_description()

        # Create service file content
        service_content = u"""[Unit]
Description={}
After=network.target

[Service]
Type=oneshot
WorkingDirectory={}
ExecStart={}
""".format(description, work_dir, ddns_command)  # fmt: skip

        # Create timer file content
        timer_content = u"""[Unit]
Description=DDNS automatic IP update timer
Requires={}

[Timer]
OnUnitActiveSec={}m
Unit={}

[Install]
WantedBy=multi-user.target
""".format(self.SERVICE_NAME, interval, self.SERVICE_NAME)  # fmt: skip

        try:
            # Write service and timer files
            write_file(self.SERVICE_PATH, service_content)
            write_file(self.TIMER_PATH, timer_content)
        except PermissionError as e:
            self.logger.debug("Permission denied when writing systemd files: %s", e)
            print("Permission denied. Please run as root or use sudo.")
            print("or use cron scheduler (with --scheduler=cron) instead.")
            return False
        except Exception as e:
            self.logger.error("Failed to write systemd files: %s", e)
            return False

        if self._systemctl("daemon-reload") and self._systemctl("enable", self.TIMER_NAME):
            self._systemctl("start", self.TIMER_NAME)
            return True
        else:
            self.logger.error("Failed to enable/start systemd timer")
            return False

    def uninstall(self):
        """Uninstall systemd timer and service"""
        self.disable()  # Stop and disable timer
        # Remove systemd files
        try:
            os.remove(self.SERVICE_PATH)
            os.remove(self.TIMER_PATH)
            self._systemctl("daemon-reload")  # Reload systemd configuration
            return True

        except PermissionError as e:
            self.logger.debug("Permission denied when removing systemd files: %s", e)
            print("Permission denied. Please run as root or use sudo.")
            return False
        except Exception as e:
            self.logger.error("Failed to remove systemd files: %s", e)
            return False

    def enable(self):
        """Enable and start systemd timer"""
        return self._systemctl("enable", self.TIMER_NAME) and self._systemctl("start", self.TIMER_NAME)

    def disable(self):
        """Disable and stop systemd timer"""
        self._systemctl("stop", self.TIMER_NAME)
        return self._systemctl("disable", self.TIMER_NAME)



# -*- coding:utf-8 -*-
"""
Unit tests for ddns.scheduler.cron module
@author: NewFuture
"""

import platform

from __init__ import patch, unittest

from ddns.scheduler.cron import CronScheduler


class TestCronScheduler(unittest.TestCase):
    """Test CronScheduler functionality"""

    def setUp(self):
        """Set up test fixtures"""
        self.scheduler = CronScheduler()

    @patch("ddns.scheduler._base.datetime")
    @patch("ddns.scheduler._base.version", "test-version")
    def test_install_with_version_and_date(self, mock_datetime):
        """Test install method includes version and date in cron entry"""
        mock_datetime.now.return_value.strftime.return_value = "2025-08-01 14:30:00"

        # Mock the methods to avoid actual system calls
        with patch("ddns.scheduler.cron.try_run") as mock_run:
            with patch.object(self.scheduler, "_update_crontab") as mock_update:
                with patch.object(self.scheduler, "_build_ddns_command") as mock_build:
                    mock_run.return_value = ""
                    mock_update.return_value = True
                    mock_build.return_value = "python3 -m ddns -c test.json"

                    result = self.scheduler.install(5, {"config": ["test.json"]})

                    self.assertTrue(result)
                    mock_update.assert_called_once()

                    # Verify the cron entry contains version and date
                    call_args = mock_update.call_args[0][0]
                    cron_entry = u"\n".join(call_args)  # fmt: skip
                    self.assertIn("# DDNS: auto-update vtest-version installed on 2025-08-01 14:30:00", cron_entry)

    def test_get_status_extracts_comments(self):
        """Test get_status method extracts comments from cron entry"""
        cron_entry = (
            '*/10 * * * * cd "/home/user" && python3 -m ddns -c test.json '
            "# DDNS: auto-update v4.0 installed on 2025-08-01 14:30:00"
        )

        with patch("ddns.scheduler.cron.try_run") as mock_run:

            def mock_command(cmd, **kwargs):
                if cmd == ["crontab", "-l"]:
                    return cron_entry
                elif cmd == ["pgrep", "-f", "cron"]:
                    return "12345"
                return None

            mock_run.side_effect = mock_command

            status = self.scheduler.get_status()

            self.assertEqual(status["scheduler"], "cron")
            self.assertTrue(status["enabled"])
            self.assertEqual(status["interval"], 10)
            self.assertEqual(status["description"], "auto-update v4.0 installed on 2025-08-01 14:30:00")

    def test_get_status_handles_missing_comment_info(self):
        """Test get_status handles cron entries without full comment info gracefully"""
        cron_entry = '*/5 * * * * cd "/home/user" && python3 -m ddns -c test.json # DDNS: auto-update'

        with patch("ddns.scheduler.cron.try_run") as mock_run:

            def mock_command(cmd, **kwargs):
                if cmd == ["crontab", "-l"]:
                    return cron_entry
                elif cmd == ["pgrep", "-f", "cron"]:
                    return None
                return None

            mock_run.side_effect = mock_command

            status = self.scheduler.get_status()

            self.assertEqual(status["scheduler"], "cron")
            self.assertTrue(status["enabled"])
            self.assertEqual(status["interval"], 5)
            self.assertEqual(status["description"], "auto-update")

    def test_version_in_cron_entry(self):
        """Test that install method includes version in cron entry"""
        with patch("ddns.scheduler._base.datetime") as mock_datetime:
            mock_datetime.now.return_value.strftime.return_value = "2025-08-01 14:30:00"

            with patch("ddns.scheduler.cron.try_run") as mock_run:
                with patch.object(self.scheduler, "_update_crontab") as mock_update:
                    with patch.object(self.scheduler, "_build_ddns_command") as mock_build:
                        mock_run.return_value = ""
                        mock_update.return_value = True
                        mock_build.return_value = "python3 -m ddns"

                        # Test that version is included in cron entry
                        with patch("ddns.scheduler._base.version", "test-version"):
                            result = self.scheduler.install(10)

                            self.assertTrue(result)
                            call_args = mock_update.call_args[0][0]
                            cron_entry = u"\n".join(call_args)  # fmt: skip
                            self.assertIn("vtest-version", cron_entry)  # Should include the version

    def test_get_status_with_no_comment(self):
        """Test get_status handles cron entries with no DDNS comment"""
        cron_entry = '*/15 * * * * cd "/home/user" && python3 -m ddns -c test.json'

        with patch("ddns.scheduler.cron.try_run") as mock_run:

            def mock_command(cmd, **kwargs):
                if cmd == ["crontab", "-l"]:
                    return cron_entry
                elif cmd == ["pgrep", "-f", "cron"]:
                    return None
                return None

            mock_run.side_effect = mock_command

            status = self.scheduler.get_status()

            self.assertEqual(status["scheduler"], "cron")
            self.assertEqual(status["enabled"], False)  # False when no DDNS line found
            # When no DDNS line is found, the method still tries to parse the empty line
            # This results in None values for interval, command, and empty string for comments
            self.assertIsNone(status.get("interval"))
            self.assertIsNone(status.get("command"))
            self.assertEqual(status.get("description"), "")

    def test_modify_cron_lines_enable_disable(self):
        """Test _modify_cron_lines method for enable and disable operations"""
        # Test enable operation on commented line
        with patch("ddns.scheduler.cron.try_run") as mock_run:
            with patch.object(self.scheduler, "_update_crontab") as mock_update:
                mock_run.return_value = "# */5 * * * * cd /path && python3 -m ddns # DDNS: auto-update"
                mock_update.return_value = True

                result = self.scheduler.enable()
                self.assertTrue(result)
                mock_update.assert_called_once()
                call_args = mock_update.call_args[0][0]
                cron_entry = u"\n".join(call_args)  # fmt: skip
                self.assertIn("*/5 * * * * cd /path && python3 -m ddns # DDNS: auto-update", cron_entry)

        # Test disable operation on active line
        with patch("ddns.scheduler.cron.try_run") as mock_run:
            with patch.object(self.scheduler, "_update_crontab") as mock_update:
                mock_run.return_value = "*/5 * * * * cd /path && python3 -m ddns # DDNS: auto-update"
                mock_update.return_value = True

                result = self.scheduler.disable()
                self.assertTrue(result)
                mock_update.assert_called_once()
                call_args = mock_update.call_args[0][0]
                cron_entry = u"\n".join(call_args)  # fmt: skip
                self.assertIn("# */5 * * * * cd /path && python3 -m ddns # DDNS: auto-update", cron_entry)

    def test_modify_cron_lines_uninstall(self):
        """Test _modify_cron_lines method for uninstall operation"""
        with patch("ddns.scheduler.cron.try_run") as mock_run:
            with patch.object(self.scheduler, "_update_crontab") as mock_update:
                mock_run.return_value = "*/5 * * * * cd /path && python3 -m ddns # DDNS: auto-update\nother cron job"
                mock_update.return_value = True

                result = self.scheduler.uninstall()
                self.assertTrue(result)
                mock_update.assert_called_once()
                call_args = mock_update.call_args[0][0]
                cron_entry = u"\n".join(call_args)  # fmt: skip
                self.assertNotIn("DDNS", cron_entry)
                self.assertIn("other cron job", cron_entry)

    @unittest.skipIf(platform.system().lower() == "windows", "Unix/Linux/macOS-specific test")
    def test_real_cron_integration(self):
        """Test real cron integration with actual system calls"""
        # Check if crontab command is available
        try:
            from ddns.util.try_run import try_run

            crontab_result = try_run(["crontab", "-l"])
            if not crontab_result:
                self.skipTest("crontab not available on this system")
        except Exception:
            self.skipTest("crontab not available on this system")

        try:
            status = self.scheduler.get_status()
            self.assertIsInstance(status, dict)
            self.assertEqual(status["scheduler"], "cron")
            self.assertIsInstance(status["installed"], bool)
        finally:
            pass

    def _setup_real_cron_test(self):
        """
        Helper method to set up real cron tests with common functionality
        """
        # Check if crontab is available first
        try:
            from ddns.util.try_run import try_run

            try_run(["crontab", "-l"])
        except Exception:
            self.skipTest("crontab not available on this system")

    def _cleanup_real_cron_test(self):
        """
        Helper method to clean up real cron tests
        """
        try:
            # Remove any test cron jobs
            if self.scheduler.is_installed():
                self.scheduler.uninstall()
        except Exception:
            pass

    @unittest.skipIf(platform.system().lower() == "windows", "Unix/Linux/macOS-specific test")
    def test_real_scheduler_methods_safe(self):
        """Test real scheduler methods that don't modify system state"""
        try:
            # Test is_installed (safe read-only operation)
            installed = self.scheduler.is_installed()
            self.assertIsInstance(installed, bool)

            # Test build command
            ddns_args = {"dns": "debug", "ipv4": ["test.example.com"]}
            command = self.scheduler._build_ddns_command(ddns_args)
            self.assertIsInstance(command, list)
            command_str = " ".join(command)
            self.assertIn("python", command_str.lower())

            # Test get status (safe read-only operation)
            status = self.scheduler.get_status()
            required_keys = ["scheduler", "installed", "enabled", "interval"]
            for key in required_keys:
                self.assertIn(key, status)

            # Test enable/disable without actual installation (should handle gracefully)
            enable_result = self.scheduler.enable()
            self.assertIsInstance(enable_result, bool)

            disable_result = self.scheduler.disable()
            self.assertIsInstance(disable_result, bool)
        finally:
            self._cleanup_real_cron_test()

    @unittest.skipIf(platform.system().lower() == "windows", "Unix/Linux/macOS-specific integration test")
    def test_real_lifecycle_comprehensive(self):
        """
        Comprehensive real-life integration test covering all lifecycle scenarios
        This combines install/enable/disable/uninstall, error handling, and crontab scenarios
        WARNING: This test modifies system state and should only run on test systems
        """
        if platform.system().lower() == "windows":
            self.skipTest("Unix/Linux/macOS-specific integration test")

        self._setup_real_cron_test()

        try:
            # ===== PHASE 1: Clean state and error handling =====
            if self.scheduler.is_installed():
                self.scheduler.uninstall()

            # Test operations on non-existent cron job
            self.assertFalse(self.scheduler.enable(), "Enable should fail for non-existent cron job")
            self.assertFalse(self.scheduler.disable(), "Disable should fail for non-existent cron job")
            self.assertFalse(self.scheduler.uninstall(), "Uninstall should fail for non-existent cron job")

            # Verify initial state
            initial_status = self.scheduler.get_status()
            self.assertEqual(initial_status["scheduler"], "cron")
            self.assertFalse(initial_status["installed"], "Cron job should not be installed initially")

            # ===== PHASE 2: Installation and validation =====
            ddns_args = {
                "dns": "debug",
                "ipv4": ["test-comprehensive.example.com"],
                "config": ["config.json"],
                "ttl": 300,
            }

            # Mock crontab commands for installation
            with patch("ddns.util.try_run.try_run") as mock_try_run, patch(
                "ddns.scheduler.cron.subprocess.check_call"
            ) as mock_check_call, patch("ddns.scheduler.cron.tempfile.mktemp", return_value="/tmp/test.cron"), patch(
                "ddns.scheduler.cron.write_file"
            ), patch("ddns.scheduler.cron.os.unlink"):

                def crontab_side_effect(command, **kwargs):
                    if command == ["crontab", "-l"]:
                        return ""  # Return empty crontab initially
                    return None

                mock_try_run.side_effect = crontab_side_effect
                mock_check_call.return_value = None  # Simulate successful crontab update
                install_result = self.scheduler.install(interval=5, ddns_args=ddns_args)
            self.assertTrue(install_result, "Installation should succeed")

            # Verify installation and crontab content - mock the get_status call
            with patch("ddns.scheduler.cron.try_run") as mock_try_run:
                # Mock crontab -l to return the installed entry
                cron_entry = '*/5 * * * * cd "/workspaces/DDNS" && python3 -m ddns --dns debug --ipv4 test-comprehensive.example.com --config config.json --ttl 300 # DDNS: Auto DDNS Update'
                mock_try_run.return_value = cron_entry
                post_install_status = self.scheduler.get_status()
            self.assertTrue(post_install_status["installed"], "Cron job should be installed")
            self.assertTrue(post_install_status["enabled"], "Cron job should be enabled")
            self.assertEqual(post_install_status["interval"], 5, "Interval should match")

            # Mock crontab content check - use the direct method instead
            with patch("ddns.scheduler.cron.try_run") as mock_try_run:
                cron_entry = '*/5 * * * * cd "/workspaces/DDNS" && python3 -m ddns --dns debug --ipv4 test-comprehensive.example.com --config config.json --ttl 300 # DDNS: Auto DDNS Update'
                mock_try_run.return_value = cron_entry

                # Use the scheduler's crontab content directly instead of importing try_run
                crontab_content = cron_entry
            self.assertIsNotNone(crontab_content, "Crontab should have content")
            if crontab_content:
                self.assertIn("DDNS", crontab_content, "Crontab should contain DDNS entry")
                self.assertIn("*/5", crontab_content, "Crontab should contain correct interval")

            # Validate cron entry format
            lines = crontab_content.strip().split("\n") if crontab_content else []
            ddns_lines = [line for line in lines if "DDNS" in line and not line.strip().startswith("#")]
            self.assertTrue(len(ddns_lines) > 0, "Should have active DDNS cron entry")

            ddns_line = ddns_lines[0]
            parts = ddns_line.split()
            self.assertTrue(len(parts) >= 5, "Cron line should have at least 5 time fields")
            self.assertEqual(parts[0], "*/5", "Should have correct minute interval")
            self.assertIn("python", ddns_line.lower(), "Should contain python command")
            if crontab_content:
                self.assertIn("debug", crontab_content, "Should contain DNS provider")

            # ===== PHASE 3: Disable/Enable cycle =====
            with patch("ddns.scheduler.cron.try_run") as mock_try_run, patch(
                "ddns.scheduler.cron.subprocess.check_call"
            ) as mock_check_call, patch("ddns.scheduler.cron.tempfile.mktemp", return_value="/tmp/test.cron"), patch(
                "ddns.scheduler.cron.write_file"
            ), patch("ddns.scheduler.cron.os.unlink"):
                # Mock crontab -l to return the installed entry (for disable operation)
                cron_entry = '*/5 * * * * cd "/workspaces/DDNS" && python3 -m ddns --dns debug --ipv4 test-comprehensive.example.com --config config.json --ttl 300 # DDNS: Auto DDNS Update'
                mock_try_run.return_value = cron_entry
                mock_check_call.return_value = None
                disable_result = self.scheduler.disable()
            self.assertTrue(disable_result, "Disable should succeed")

            # Check status after disable
            with patch("ddns.scheduler.cron.try_run") as mock_try_run:
                # Mock disabled crontab (entry is commented out)
                disabled_entry = '# */5 * * * * cd "/workspaces/DDNS" && python3 -m ddns --dns debug --ipv4 test-comprehensive.example.com --config config.json --ttl 300 # DDNS: Auto DDNS Update'
                mock_try_run.return_value = disabled_entry
                post_disable_status = self.scheduler.get_status()
            self.assertTrue(post_disable_status["installed"], "Should still be installed after disable")
            self.assertFalse(post_disable_status["enabled"], "Should be disabled")

            # Verify cron entry is commented out - simulate the disabled state
            disabled_crontab = disabled_entry
            if disabled_crontab:
                disabled_lines = [line for line in disabled_crontab.split("\n") if "DDNS" in line]
                self.assertTrue(
                    all(line.strip().startswith("#") for line in disabled_lines),
                    "All DDNS lines should be commented when disabled",
                )

            # Enable operation
            with patch("ddns.scheduler.cron.try_run") as mock_try_run, patch(
                "ddns.scheduler.cron.subprocess.check_call"
            ) as mock_check_call, patch("ddns.scheduler.cron.tempfile.mktemp", return_value="/tmp/test.cron"), patch(
                "ddns.scheduler.cron.write_file"
            ), patch("ddns.scheduler.cron.os.unlink"):
                # Mock crontab -l to return the disabled entry (for enable operation)
                disabled_entry = '# */5 * * * * cd "/workspaces/DDNS" && python3 -m ddns --dns debug --ipv4 test-comprehensive.example.com --config config.json --ttl 300 # DDNS: Auto DDNS Update'
                mock_try_run.return_value = disabled_entry
                mock_check_call.return_value = None
                enable_result = self.scheduler.enable()
            self.assertTrue(enable_result, "Enable should succeed")

            # Check status after enable
            with patch("ddns.scheduler.cron.try_run") as mock_try_run:
                # Mock enabled crontab (entry is uncommented)
                enabled_entry = '*/5 * * * * cd "/workspaces/DDNS" && python3 -m ddns --dns debug --ipv4 test-comprehensive.example.com --config config.json --ttl 300 # DDNS: Auto DDNS Update'
                mock_try_run.return_value = enabled_entry
                post_enable_status = self.scheduler.get_status()
            self.assertTrue(post_enable_status["installed"], "Should still be installed after enable")
            self.assertTrue(post_enable_status["enabled"], "Should be enabled")

            # ===== PHASE 4: Duplicate installation test =====
            with patch("ddns.scheduler.cron.try_run") as mock_try_run, patch(
                "ddns.scheduler.cron.subprocess.check_call"
            ) as mock_check_call, patch("ddns.scheduler.cron.tempfile.mktemp", return_value="/tmp/test.cron"), patch(
                "ddns.scheduler.cron.write_file"
            ), patch("ddns.scheduler.cron.os.unlink"):
                enabled_entry = '*/5 * * * * cd "/workspaces/DDNS" && python3 -m ddns --dns debug --ipv4 test-comprehensive.example.com --config config.json --ttl 300 # DDNS: Auto DDNS Update'
                mock_try_run.return_value = enabled_entry
                mock_check_call.return_value = None
                duplicate_install = self.scheduler.install(interval=5, ddns_args=ddns_args)
            self.assertIsInstance(duplicate_install, bool, "Duplicate install should return boolean")

            with patch("ddns.scheduler.cron.try_run") as mock_try_run:
                enabled_entry = '*/5 * * * * cd "/workspaces/DDNS" && python3 -m ddns --dns debug --ipv4 test-comprehensive.example.com --config config.json --ttl 300 # DDNS: Auto DDNS Update'
                mock_try_run.return_value = enabled_entry
                status_after_duplicate = self.scheduler.get_status()
            self.assertTrue(status_after_duplicate["installed"], "Should remain installed after duplicate")

            # ===== PHASE 5: Uninstall and verification =====
            with patch("ddns.scheduler.cron.try_run") as mock_try_run, patch(
                "ddns.scheduler.cron.subprocess.check_call"
            ) as mock_check_call, patch("ddns.scheduler.cron.tempfile.mktemp", return_value="/tmp/test.cron"), patch(
                "ddns.scheduler.cron.write_file"
            ), patch("ddns.scheduler.cron.os.unlink"):
                enabled_entry = '*/5 * * * * cd "/workspaces/DDNS" && python3 -m ddns --dns debug --ipv4 test-comprehensive.example.com --config config.json --ttl 300 # DDNS: Auto DDNS Update'
                mock_try_run.return_value = enabled_entry
                mock_check_call.return_value = None
                uninstall_result = self.scheduler.uninstall()
            self.assertTrue(uninstall_result, "Uninstall should succeed")

            # Check final status after uninstall
            with patch("ddns.scheduler.cron.try_run") as mock_try_run:
                mock_try_run.return_value = ""  # Empty crontab
                final_status = self.scheduler.get_status()
                is_installed = self.scheduler.is_installed()
            self.assertFalse(final_status["installed"], "Should not be installed after uninstall")
            self.assertFalse(is_installed, "is_installed() should return False")

            # Verify complete removal from crontab
            from ddns.util.try_run import try_run

            final_crontab = try_run(["crontab", "-l"])
            if final_crontab:
                self.assertNotIn("DDNS", final_crontab, "DDNS should be completely removed")
        finally:
            self._cleanup_real_cron_test()


if __name__ == "__main__":
    unittest.main()



# coding=utf-8
"""
DNSPOD API
@doc: https://docs.dnspod.cn/api/
@author: NewFuture
"""

from ._base import BaseProvider, TYPE_FORM


class DnspodProvider(BaseProvider):
    """
    DNSPOD API
    DNSPOD 接口解析操作库
    """

    endpoint = "https://dnsapi.cn"
    content_type = TYPE_FORM

    DefaultLine = "默认"

    def _request(self, action, extra=None, **params):
        # type: (str, dict | None, **(str | int | bytes | bool | None)) -> dict
        """
        发送请求数据

        Send request to DNSPod API.
        Args:
            action (str): API 动作/Action
            extra (dict|None): 额外参数/Extra params
            params (dict): 其它参数/Other params
        Returns:
            dict: 响应数据/Response data
        """
        # 过滤掉None参数
        if extra:
            params.update(extra)
        params = {k: v for k, v in params.items() if v is not None}
        params.update({"login_token": "{0},{1}".format(self.id, self.token), "format": "json"})
        data = self._http("POST", "/" + action, body=params)
        if data and data.get("status", {}).get("code") == "1":  # 请求成功
            return data
        else:  # 请求失败
            error_msg = "Unknown error"
            if data and isinstance(data, dict):
                error_msg = data.get("status", {}).get("message", "Unknown error")
            self.logger.warning("DNSPod API error: %s", error_msg)
            return data

    def _create_record(self, zone_id, subdomain, main_domain, value, record_type, ttl, line, extra):
        # type: (str, str, str, str, str, int | str | None, str | None, dict) -> bool
        """https://docs.dnspod.cn/api/add-record/"""
        res = self._request(
            "Record.Create",
            extra=extra,
            domain_id=zone_id,
            sub_domain=subdomain,
            value=value,
            record_type=record_type,
            record_line=line or self.DefaultLine,
            ttl=ttl,
        )
        record = res and res.get("record")
        if record:  # 记录创建成功
            self.logger.info("Record created: %s", record)
            return True
        else:  # 记录创建失败
            self.logger.error("Failed to create record: %s", res)
        return False

    def _update_record(self, zone_id, old_record, value, record_type, ttl, line, extra):
        # type: (str, dict, str, str, int | str | None, str | None, dict) -> bool
        """https://docs.dnspod.cn/api/modify-records/"""
        record_line = (line or old_record.get("line") or self.DefaultLine).replace("Default", "default")
        res = self._request(
            "Record.Modify",
            domain_id=zone_id,
            record_id=old_record.get("id"),
            sub_domain=old_record.get("name"),
            record_type=record_type,
            value=value,
            record_line=record_line,
            extra=extra,
        )
        record = res and res.get("record")
        if record:  # 记录更新成功
            self.logger.debug("Record updated: %s", record)
            return True
        else:  # 记录更新失败
            self.logger.error("Failed to update record: %s", res)
            return False

    def _query_zone_id(self, domain):
        # type: (str) -> str | None
        """查询域名信息 https://docs.dnspod.cn/api/domain-info/"""
        res = self._request("Domain.Info", domain=domain)
        if res and isinstance(res, dict):
            return res.get("domain", {}).get("id")
        return None

    def _query_record(self, zone_id, subdomain, main_domain, record_type, line, extra):
        # type: (str, str, str, str, str | None, dict) -> dict | None
        """查询记录 list 然后逐个查找 https://docs.dnspod.cn/api/record-list/"""
        res = self._request("Record.List", domain_id=zone_id, sub_domain=subdomain, record_type=record_type, line=line)
        # length="3000"
        records = res.get("records", [])
        n = len(records)
        if not n:
            self.logger.warning("No record found for [%s] %s<%s>(line: %s)", zone_id, subdomain, record_type, line)
            return None
        if n > 1:
            self.logger.warning("%d records found for %s<%s>(%s):\n %s", n, subdomain, record_type, line, records)
            return next((r for r in records if r.get("name") == subdomain), None)
        return records[0]



# coding=utf-8
"""
测试 provider 基类的代理列表功能
Test provider base class proxy list functionality
"""

from base_test import BaseProviderTestCase, patch, unittest
from ddns.provider._base import SimpleProvider
from ddns.util.http import HttpResponse


class TestSimpleProvider(SimpleProvider):
    """测试用的简单Provider实现"""

    endpoint = "https://api.example.com"

    def set_record(self, domain, value, record_type="A", ttl=None, line=None, **extra):
        # type: (str, str, str, str | int | None, str | None, **object) -> bool
        """简单的set_record实现用于测试"""
        return self._http("POST", "/set", body={"domain": domain, "value": value})


class TestProviderProxyList(BaseProviderTestCase):
    """测试Provider代理列表功能"""

    def setUp(self):
        """设置测试环境"""
        super(TestProviderProxyList, self).setUp()
        # 创建一个包含代理列表的provider
        self.proxy_list = ["http://proxy1:8080", "http://proxy2:8080", None]

    @patch("ddns.provider._base.request")
    def test_provider_http_with_proxy_list(self, mock_request):
        """测试Provider使用代理列表发送HTTP请求"""
        # 模拟成功响应
        mock_response = HttpResponse(200, "OK", {}, '{"status": "success", "data": "test"}')
        mock_request.return_value = mock_response

        # 创建provider并设置代理列表
        provider = TestSimpleProvider(self.authid, self.token, proxy=self.proxy_list)

        # 调用_http方法
        result = provider._http("GET", "/test")

        # 验证结果（应该是解析后的JSON）
        self.assertEqual(result["status"], "success")

        # 验证request被正确调用，传递了proxies参数
        mock_request.assert_called_once()
        call_args = mock_request.call_args
        self.assertEqual(call_args[1]["proxies"], self.proxy_list)

    @patch("ddns.provider._base.request")
    def test_provider_http_with_single_proxy_backward_compatibility(self, mock_request):
        """测试Provider单个代理列表"""
        # 模拟成功响应
        mock_response = HttpResponse(200, "OK", {}, '{"status": "success", "data": "test"}')
        mock_request.return_value = mock_response

        # 创建provider并设置单个代理列表
        single_proxy_list = ["http://single-proxy:8080"]
        provider = TestSimpleProvider(self.authid, self.token, proxy=single_proxy_list)

        # 调用_http方法
        result = provider._http("GET", "/test")

        # 验证结果
        self.assertEqual(result["status"], "success")

        # 验证request被正确调用
        mock_request.assert_called_once()
        call_args = mock_request.call_args
        self.assertEqual(call_args[1]["proxies"], single_proxy_list)

    @patch("ddns.provider._base.request")
    def test_provider_http_no_proxy(self, mock_request):
        """测试Provider没有代理时的默认行为"""
        # 模拟成功响应
        mock_response = HttpResponse(200, "OK", {}, '{"status": "success", "data": "test"}')
        mock_request.return_value = mock_response

        # 创建provider不设置代理
        provider = TestSimpleProvider(self.authid, self.token)

        # 调用_http方法
        result = provider._http("GET", "/test")

        # 验证结果
        self.assertEqual(result["status"], "success")

        # 验证request被正确调用
        mock_request.assert_called_once()
        call_args = mock_request.call_args
        self.assertEqual(call_args[1]["proxies"], None)

    @patch("ddns.provider._base.request")
    def test_provider_http_empty_proxy_list(self, mock_request):
        """测试Provider空代理列表时的默认行为"""
        # 模拟成功响应
        mock_response = HttpResponse(200, "OK", {}, '{"status": "success", "data": "test"}')
        mock_request.return_value = mock_response

        # 创建provider设置空代理列表
        provider = TestSimpleProvider(self.authid, self.token, proxy=[])

        # 调用_http方法
        result = provider._http("GET", "/test")

        # 验证结果
        self.assertEqual(result["status"], "success")

        # 验证request被正确调用
        mock_request.assert_called_once()
        call_args = mock_request.call_args
        self.assertEqual(call_args[1]["proxies"], [])

    @patch("ddns.provider._base.request")
    def test_provider_http_request_failure_handling(self, mock_request):
        """测试Provider处理请求失败的情况"""
        # 模拟请求失败
        mock_request.side_effect = RuntimeError("All proxies failed")

        # 创建provider
        provider = TestSimpleProvider(self.authid, self.token, proxy=self.proxy_list)

        # 调用_http方法应该抛出异常
        with self.assertRaises(RuntimeError) as context:
            provider._http("GET", "/test")

        self.assertIn("All proxies failed", str(context.exception))

    def test_provider_initialization_with_proxy_types(self):
        """测试Provider初始化时不同代理参数类型的处理"""
        # 测试代理列表
        proxy_list = ["http://proxy1:8080", "http://proxy2:8080", None]
        provider1 = TestSimpleProvider(self.authid, self.token, proxy=proxy_list)
        self.assertEqual(provider1._proxy, proxy_list)

        # 测试单项代理列表
        single_proxy_list = ["http://proxy:8080"]
        provider2 = TestSimpleProvider(self.authid, self.token, proxy=single_proxy_list)
        self.assertEqual(provider2._proxy, single_proxy_list)

        # 测试None代理
        provider3 = TestSimpleProvider(self.authid, self.token, proxy=None)
        self.assertEqual(provider3._proxy, None)

        # 测试空列表代理
        provider4 = TestSimpleProvider(self.authid, self.token, proxy=[])
        self.assertEqual(provider4._proxy, [])


if __name__ == "__main__":
    unittest.main()



# coding=utf-8
"""
AliESA API
阿里云边缘安全加速(ESA) DNS 解析操作库
@author: NewFuture, GitHub Copilot
"""

from time import strftime

from ._base import TYPE_JSON, join_domain
from .alidns import AliBaseProvider


class AliesaProvider(AliBaseProvider):
    """阿里云边缘安全加速(ESA) DNS Provider"""

    endpoint = "https://esa.cn-hangzhou.aliyuncs.com"
    api_version = "2024-09-10"  # ESA API版本
    content_type = TYPE_JSON
    remark = "Managed by DDNS %s" % strftime("%Y-%m-%d %H:%M:%S")

    def _query_zone_id(self, domain):
        # type: (str) -> str | None
        """
        查询站点ID
        https://help.aliyun.com/zh/edge-security-acceleration/esa/api-esa-2024-09-10-listsites
        """
        res = self._request(method="GET", action="ListSites", SiteName=domain, PageSize=500)
        sites = res.get("Sites", [])

        for site in sites:
            if site.get("SiteName") == domain:
                site_id = site.get("SiteId")
                self.logger.debug("Found site ID %s for domain %s", site_id, domain)
                return site_id

        self.logger.error("Site not found for domain: %s", domain)
        return None

    def _query_record(self, zone_id, subdomain, main_domain, record_type, line, extra):
        # type: (str, str, str, str, str | None, dict) -> dict | None
        """
        查询DNS记录
        https://help.aliyun.com/zh/edge-security-acceleration/esa/api-esa-2024-09-10-listrecords
        """
        full_domain = join_domain(subdomain, main_domain)
        res = self._request(
            method="GET",
            action="ListRecords",
            SiteId=int(zone_id),
            RecordName=full_domain,
            Type=self._get_type(record_type),
            RecordMatchType="exact",  # 精确匹配
            PageSize=100,
        )

        records = res.get("Records", [])
        if len(records) == 0:
            self.logger.warning("No records found for [%s] with %s <%s>", zone_id, full_domain, record_type)
            return None

        # 返回第一个匹配的记录
        record = records[0]
        self.logger.debug("Found record: %s", record)
        return record

    def _create_record(self, zone_id, subdomain, main_domain, value, record_type, ttl, line, extra):
        # type: (str, str, str, str, str, int | str | None, str | None, dict) -> bool
        """
        创建DNS记录
        https://help.aliyun.com/zh/edge-security-acceleration/esa/api-esa-2024-09-10-createrecord
        """
        full_domain = join_domain(subdomain, main_domain)
        extra["Comment"] = extra.get("Comment", self.remark)
        extra["BizName"] = extra.get("BizName", "web")
        extra["Proxied"] = extra.get("Proxied", True)
        data = self._request(
            method="POST",
            action="CreateRecord",
            SiteId=int(zone_id),
            RecordName=full_domain,
            Type=self._get_type(record_type),
            Data={"Value": value},
            Ttl=ttl or 1,
            **extra
        )  # fmt: skip

        if data and data.get("RecordId"):
            self.logger.info("Record created: %s", data)
            return True

        self.logger.error("Failed to create record: %s", data)
        return False

    def _update_record(self, zone_id, old_record, value, record_type, ttl, line, extra):
        # type: (str, dict, str, str, int | str | None, str | None, dict) -> bool
        """
        更新DNS记录
        https://help.aliyun.com/zh/edge-security-acceleration/esa/api-esa-2024-09-10-updaterecord
        """
        # 检查是否需要更新
        if (
            old_record.get("Data", {}).get("Value") == value
            and old_record.get("RecordType") == self._get_type(record_type)
            and (not ttl or old_record.get("Ttl") == ttl)
        ):
            self.logger.warning("No changes detected, skipping update for record: %s", old_record.get("RecordName"))
            return True

        extra["Comment"] = extra.get("Comment", self.remark)
        extra["Proxied"] = extra.get("Proxied", old_record.get("Proxied"))
        data = self._request(
            method="POST",
            action="UpdateRecord",
            RecordId=old_record.get("RecordId"),
            Data={"Value": value},
            Ttl=ttl,
            **extra
        )  # fmt: skip

        if data and data.get("RecordId"):
            self.logger.info("Record updated: %s", data)
            return True

        self.logger.error("Failed to update record: %s", data)
        return False

    def _get_type(self, record_type):
        # type: (str) -> str
        return "A/AAAA" if record_type in ("A", "AAAA") else record_type



# coding=utf-8
"""
HuaweiDNS API
华为DNS解析操作库
@author: NewFuture
"""

from time import gmtime, strftime

from ._base import TYPE_JSON, BaseProvider, encode_params, join_domain
from ._signature import hmac_sha256_authorization, sha256_hash


class HuaweiDNSProvider(BaseProvider):
    endpoint = "https://dns.myhuaweicloud.com"
    content_type = TYPE_JSON
    algorithm = "SDK-HMAC-SHA256"

    def _validate(self):
        self.logger.warning(
            "华为云 DNS 缺少充分的真实环境测试，请及时在 GitHub Issues 中反馈: %s",
            "https://github.com/NewFuture/DDNS/issues",
        )
        super(HuaweiDNSProvider, self)._validate()

    def _request(self, method, path, **params):
        """
        https://support.huaweicloud.com/api-dns/zh-cn_topic_0037134406.html
        https://support.huaweicloud.com/devg-apisign/api-sign-algorithm-002.html
        https://support.huaweicloud.com/devg-apisign/api-sign-algorithm-003.html
        https://support.huaweicloud.com/devg-apisign/api-sign-algorithm-004.html
        """
        # type: (str, str, **Any) -> dict
        params = {k: v for k, v in params.items() if v is not None}
        if method.upper() == "GET" or method.upper() == "DELETE":
            query = encode_params(params)
            body = ""
        else:
            query = ""
            body = self._encode_body(params)

        now = strftime("%Y%m%dT%H%M%SZ", gmtime())
        headers = {
            "content-type": self.content_type,
            "host": self.endpoint.split("://", 1)[1].strip("/"),
            "X-Sdk-Date": now,
        }

        # 使用通用签名函数
        body_hash = sha256_hash(body)
        # 华为云需要在签名时使用带尾斜杠的路径
        sign_path = path if path.endswith("/") else path + "/"
        authorization_format = "%s Access=%s, SignedHeaders={SignedHeaders}, Signature={Signature}" % (
            self.algorithm,
            self.id,
        )
        authorization = hmac_sha256_authorization(
            secret_key=self.token,
            method=method,
            path=sign_path,
            query=query,
            headers=headers,
            body_hash=body_hash,
            signing_string_format=self.algorithm + "\n" + now + "\n{HashedCanonicalRequest}",
            authorization_format=authorization_format,
        )
        headers["Authorization"] = authorization

        # 使用原始路径发送实际请求
        path = "{}?{}".format(path, query) if query else path
        data = self._http(method, path, headers=headers, body=body)
        return data

    def _query_zone_id(self, domain):
        """https://support.huaweicloud.com/api-dns/dns_api_62003.html"""
        domain = domain + "." if not domain.endswith(".") else domain
        data = self._request("GET", "/v2/zones", search_mode="equal", limit=500, name=domain)
        zones = data.get("zones", [])
        zone = next((z for z in zones if domain == z.get("name")), None)
        zoneid = zone and zone["id"]
        return zoneid

    def _query_record(self, zone_id, subdomain, main_domain, record_type, line, extra):
        """
        v2.1 https://support.huaweicloud.com/api-dns/dns_api_64004.html
        v2 https://support.huaweicloud.com/api-dns/ListRecordSetsByZone.html
        """
        domain = join_domain(subdomain, main_domain) + "."
        data = self._request(
            "GET",
            "/v2.1/zones/" + zone_id + "/recordsets",
            limit=500,
            name=domain,
            type=record_type,
            line_id=line,
            search_mode="equal",
        )
        records = data.get("recordsets", [])
        record = next((r for r in records if r.get("name") == domain and r.get("type") == record_type), None)
        return record

    def _create_record(self, zone_id, subdomain, main_domain, value, record_type, ttl, line, extra):
        """
        v2.1 https://support.huaweicloud.com/api-dns/dns_api_64001.html
        v2 https://support.huaweicloud.com/api-dns/CreateRecordSet.html
        """
        domain = join_domain(subdomain, main_domain) + "."
        extra["description"] = extra.get("description", self.remark)
        res = self._request(
            "POST",
            "/v2.1/zones/" + zone_id + "/recordsets",
            name=domain,
            type=record_type,
            records=[value],
            ttl=ttl,
            line=line,
            **extra
        )  # fmt: skip
        if res and res.get("id"):
            self.logger.info("Record created: %s", res)
            return True
        self.logger.warning("Failed to create record: %s", res)
        return False

    def _update_record(self, zone_id, old_record, value, record_type, ttl, line, extra):
        """https://support.huaweicloud.com/api-dns/UpdateRecordSets.html"""
        extra["description"] = extra.get("description", self.remark)
        # Note: The v2.1 update API does not support the line parameter in the request body
        # The line parameter is returned in the response but cannot be modified
        res = self._request(
            "PUT",
            "/v2.1/zones/" + zone_id + "/recordsets/" + old_record["id"],
            name=old_record["name"],
            type=record_type,
            records=[value],
            ttl=ttl if ttl is not None else old_record.get("ttl"),
            **extra
        )  # fmt: skip
        if res and res.get("id"):
            self.logger.info("Record updated: %s", res)
            return True
        self.logger.warning("Failed to update record: %s", res)
        return False



# coding=utf-8
"""
Custom Callback API
自定义回调接口解析操作库

@author: 老周部落, NewFuture
"""

from ._base import TYPE_JSON, SimpleProvider
from time import time
from json import loads as jsondecode


class CallbackProvider(SimpleProvider):
    """
    通用自定义回调 Provider，支持 GET/POST 任意接口。
    Generic custom callback provider, supports GET/POST arbitrary API.
    """

    endpoint = ""  # CallbackProvider uses id as URL, no fixed API endpoint
    content_type = TYPE_JSON
    decode_response = False  # Callback response is not JSON, it's a custom response

    def set_record(self, domain, value, record_type="A", ttl=None, line=None, **extra):
        """
        发送自定义回调请求，支持 GET/POST
        Send custom callback request, support GET/POST
        """
        self.logger.info("%s => %s(%s)", domain, value, record_type)
        url = self.id  # 直接用 id 作为 url
        token = self.token  # token 作为 POST 参数
        extra.update(
            {
                "__DOMAIN__": domain,
                "__RECORDTYPE__": record_type,
                "__TTL__": ttl,
                "__IP__": value,
                "__TIMESTAMP__": time(),
                "__LINE__": line,
            }
        )
        url = self._replace_vars(url, extra)
        method, params = "GET", None
        if token:
            # 如果有 token，使用 POST 方法
            method = "POST"
            # POST 方式，token 作为 POST 参数
            params = token if isinstance(token, dict) else jsondecode(token)
            for k, v in params.items():
                if hasattr(v, "replace"):  # 判断是否支持字符串替换, 兼容py2,py3
                    params[k] = self._replace_vars(v, extra)

        try:
            res = self._http(method, url, body=params)
            if res is not None:
                self.logger.info("Callback result: %s", res)
                return True
            else:
                self.logger.warning("Callback received empty response.")
        except Exception as e:
            self.logger.error("Callback failed: %s", e)
        return False

    def _replace_vars(self, string, mapping):
        # type: (str, dict) -> str
        """
        替换字符串中的变量为实际值
        Replace variables in string with actual values
        """
        for k, v in mapping.items():
            string = string.replace(k, str(v))
        return string

    def _validate(self):
        # CallbackProvider uses id as URL, not as regular ID
        if self.endpoint or (not self.id or "://" not in self.id):
            # 如果 endpoint 已经设置，或者 id 不是有效的 URL，则抛出异常
            self.logger.critical("endpoint [%s] or id [%s] 必须是有效的URL", self.endpoint, self.id)
            raise ValueError("endpoint or id must be configured with URL")



# coding=utf-8
"""
Tencent Cloud EdgeOne API
腾讯云 EdgeOne (边缘安全速平台) API
API Documentation: https://cloud.tencent.com/document/api/1552/80731
@author: NewFuture
"""

from ddns.provider._base import join_domain
from .tencentcloud import TencentCloudProvider


class EdgeOneProvider(TencentCloudProvider):
    """
    腾讯云 EdgeOne API 提供商
    Tencent Cloud EdgeOne API Provider
    """

    endpoint = "https://teo.tencentcloudapi.com"
    # 腾讯云 EdgeOne API 配置
    service = "teo"
    version_date = "2022-09-01"

    def _query_zone_id(self, domain):
        # type: (str) -> str | None
        """查询域名的加速域名信息获取 ZoneId https://cloud.tencent.com/document/api/1552/80713"""
        # 首先尝试直接查找域名
        filters = [{"Name": "zone-name", "Values": [domain], "Fuzzy": False}]  # type: Any
        response = self._request("DescribeZones", Filters=filters)

        if response and "Zones" in response:
            for zone in response.get("Zones", []):
                if zone.get("ZoneName") == domain:
                    zone_id = zone.get("ZoneId")
                    if zone_id:
                        self.logger.debug("Found acceleration domain %s with Zone ID: %s", domain, zone_id)
                        return zone_id

        self.logger.debug("Acceleration domain not found for: %s", domain)
        return None

    def _query_record(self, zone_id, subdomain, main_domain, record_type, line, extra):
        # type: (str, str, str, str, str | None, dict) -> dict | None
        """查询加速域名信息 https://cloud.tencent.com/document/api/1552/86336"""
        domain = join_domain(subdomain, main_domain)
        filters = [{"Name": "domain-name", "Values": [domain], "Fuzzy": False}]  # type: Any
        response = self._request("DescribeAccelerationDomains", ZoneId=zone_id, Filters=filters)

        if response and "AccelerationDomains" in response:
            for domain_info in response.get("AccelerationDomains", []):
                if domain_info.get("DomainName") == domain:
                    self.logger.debug("Found acceleration domain: %s", domain_info)
                    return domain_info

        self.logger.warning("No acceleration domain found for: %s, response: %s", domain, response)
        return None

    def _create_record(self, zone_id, subdomain, main_domain, value, record_type, ttl, line, extra):
        # type: (str, str, str, str, str, int, str | None, dict) -> bool
        """创建新的加速域名记录 https://cloud.tencent.com/document/api/1552/86338"""
        domain = join_domain(subdomain, main_domain)
        origin = {"OriginType": "IP_DOMAIN", "Origin": value}  # type: Any
        res = self._request("CreateAccelerationDomain", ZoneId=zone_id, DomainName=domain, OriginInfo=origin, **extra)
        if res:
            self.logger.info("Acceleration domain created (%s)", res.get("RequestId"))
            return True

        self.logger.error("Failed to create acceleration domain, response: %s", res)
        return False

    def _update_record(self, zone_id, old_record, value, record_type, ttl, line, extra):
        """更新加速域名的源站 IP 地址 https://cloud.tencent.com/document/api/1552/86335"""
        domain = old_record.get("DomainName")
        # 构建源站信息
        backup = old_record.get("OriginDetail", {}).get("BackupOrigin", "")
        origin = {"OriginType": "IP_DOMAIN", "Origin": value, "BackupOrigin": backup}  # type: Any
        response = self._request("ModifyAccelerationDomain", ZoneId=zone_id, DomainName=domain, OriginInfo=origin)

        if response:
            self.logger.info("Acceleration domain updated (%s)", response.get("RequestId"))
            return True
        self.logger.error("Failed to update acceleration domain origin, response: %s", response)
        return False



#!/usr/bin/env python
# coding=utf-8
"""
BaseProvider 单元测试
支持 Python 2.7 和 Python 3
"""

from base_test import BaseProviderTestCase, unittest
from ddns.provider._base import BaseProvider, encode_params


class _TestProvider(BaseProvider):
    """测试用的具体Provider实现"""

    endpoint = "https://api.example.com"

    def __init__(self, id="test_id", token="test_token_123456789", **options):
        super(_TestProvider, self).__init__(id, token, **options)
        self._test_zone_data = {"example.com": "zone123", "test.com": "zone456"}
        self._test_records = {}

    def _query_zone_id(self, domain):
        return self._test_zone_data.get(domain)

    def _query_record(self, zone_id, subdomain, main_domain, record_type, line=None, extra=None):
        key = "{}-{}-{}".format(zone_id, subdomain, record_type)
        return self._test_records.get(key)

    def _create_record(self, zone_id, subdomain, main_domain, value, record_type, ttl=None, line=None, extra=None):
        key = "{}-{}-{}".format(zone_id, subdomain, record_type)
        self._test_records[key] = {"id": "rec123", "name": subdomain, "value": value, "type": record_type}
        return True

    def _update_record(self, zone_id, old_record, value, record_type, ttl=None, line=None, extra=None):
        old_record["value"] = value
        return True


class TestBaseProvider(BaseProviderTestCase):
    """BaseProvider 测试类"""

    def setUp(self):
        """测试初始化"""
        super(TestBaseProvider, self).setUp()
        self.provider = _TestProvider()

    def test_init_success(self):
        """测试正常初始化"""
        provider = _TestProvider("test_id", "test_token")
        self.assertEqual(provider.id, "test_id")
        self.assertEqual(provider.token, "test_token")
        self.assertIsNotNone(provider.logger)
        self.assertEqual(provider._proxy, None)  # proxy 初始化为 None
        self.assertEqual(provider._zone_map, {})

    def test_validate_missing_id(self):
        """测试缺少id的验证"""
        with self.assertRaises(ValueError) as cm:
            _TestProvider("", "token")
        self.assertIn("id must be configured", str(cm.exception))

    def test_validate_missing_token(self):
        """测试缺少token的验证"""
        with self.assertRaises(ValueError) as cm:
            _TestProvider("id", "")
        self.assertIn("token must be configured", str(cm.exception))

    def test_init_with_endpoint_override(self):
        """测试使用endpoint参数覆盖默认API"""
        custom_endpoint = "https://custom.api.com"
        provider = _TestProvider("test_id", "test_token", endpoint=custom_endpoint)
        self.assertEqual(provider.endpoint, custom_endpoint)
        self.assertEqual(provider.id, "test_id")
        self.assertEqual(provider.token, "test_token")

    def test_init_without_endpoint_uses_default(self):
        """测试不提供endpoint时使用默认API"""
        provider = _TestProvider("test_id", "test_token")
        self.assertEqual(provider.endpoint, "https://api.example.com")  # 使用类级别的默认值
        self.assertEqual(provider.id, "test_id")
        self.assertEqual(provider.token, "test_token")

    def test_init_with_empty_endpoint_ignored(self):
        """测试空endpoint参数被忽略"""
        provider = _TestProvider("test_id", "test_token", endpoint="")
        self.assertEqual(provider.endpoint, "https://api.example.com")  # 使用类级别的默认值

        provider = _TestProvider("test_id", "test_token", endpoint=None)
        self.assertEqual(provider.endpoint, "https://api.example.com")  # 使用类级别的默认值

    def test_remark_exists_and_format(self):
        """测试remark存在且格式正确"""
        provider = _TestProvider("test_id", "test_token")
        self.assertTrue(hasattr(provider, "remark"))
        self.assertIsInstance(provider.remark, str)
        self.assertGreater(len(provider.remark), 0)
        # 检查是否包含基本的说明信息
        self.assertIn("DDNS", provider.remark)

    def test_endpoint_priority_over_class_api(self):
        """测试endpoint参数优先级高于类级别API"""

        # 创建一个有不同默认API的测试类
        class _CustomAPIProvider(_TestProvider):
            endpoint = "https://different.api.com"

        # 不使用endpoint - 应该使用类级别的API
        provider1 = _CustomAPIProvider("id", "token")
        self.assertEqual(provider1.endpoint, "https://different.api.com")

        # 使用endpoint - 应该覆盖类级别的API
        custom_endpoint = "https://override.api.com"
        provider2 = _CustomAPIProvider("id", "token", endpoint=custom_endpoint)
        self.assertEqual(provider2.endpoint, custom_endpoint)

    def test_get_zone_id_from_cache(self):
        """测试从缓存获取zone_id"""
        self.provider._zone_map["cached.com"] = "cached_zone"
        zone_id = self.provider.get_zone_id("cached.com")
        self.assertEqual(zone_id, "cached_zone")

    def test_get_zone_id_query_and_cache(self):
        """测试查询并缓存zone_id"""
        zone_id = self.provider.get_zone_id("example.com")
        self.assertEqual(zone_id, "zone123")
        self.assertEqual(self.provider._zone_map["example.com"], "zone123")

    def test_split_custom_domain_with_tilde(self):
        """测试用~分隔的自定义域名"""
        from ddns.provider._base import _split_custom_domain

        sub, main = _split_custom_domain("www~example.com")
        self.assertEqual(sub, "www")
        self.assertEqual(main, "example.com")

    def test_split_custom_domain_with_plus(self):
        """测试用+分隔的自定义域名"""
        from ddns.provider._base import _split_custom_domain

        sub, main = _split_custom_domain("api+test.com")
        self.assertEqual(sub, "api")
        self.assertEqual(main, "test.com")

    def test_split_custom_domain_no_separator(self):
        """测试没有分隔符的域名"""
        from ddns.provider._base import _split_custom_domain

        sub, main = _split_custom_domain("example.com")
        self.assertIsNone(sub)
        self.assertEqual(main, "example.com")

    def test_join_domain_normal(self):
        """测试正常合并域名"""
        from ddns.provider._base import join_domain

        domain = join_domain("www", "example.com")
        self.assertEqual(domain, "www.example.com")

    def test_join_domain_empty_sub(self):
        """测试空子域名合并"""
        from ddns.provider._base import join_domain

        domain = join_domain("", "example.com")
        self.assertEqual(domain, "example.com")

        domain = join_domain("@", "example.com")
        self.assertEqual(domain, "example.com")

    def test_encode_dict(self):
        """测试编码字典参数"""
        params = {"key1": "value1", "key2": "value2"}
        result = encode_params(params)
        # 由于字典顺序可能不同，我们检查包含关系
        self.assertIn("key1=value1", result)
        self.assertIn("key2=value2", result)

    def test_encode_none(self):
        """测试编码None参数"""
        result = encode_params(None)
        self.assertEqual(result, "")

    def test_mask_sensitive_data_empty(self):
        """测试空数据打码"""
        result = self.provider._mask_sensitive_data("")
        self.assertEqual(result, "")

        result = self.provider._mask_sensitive_data(None)
        self.assertEqual(result, None)

    def test_mask_sensitive_data_long_token(self):
        """测试长token的打码"""
        data = "token=test_token_123456789&other=value"
        result = self.provider._mask_sensitive_data(data)
        expected = "token=te***89&other=value"
        self.assertEqual(result, expected)

    def test_set_record_create(self):
        """测试创建记录"""
        result = self.provider.set_record("www~example.com", "1.2.3.4", "A")
        self.assertTrue(result)
        # 验证记录是否被创建
        record = self.provider._query_record("zone123", "www", "example.com", "A", None, {})
        self.assertIsNotNone(record)
        if record:  # Type narrowing for mypy
            self.assertEqual(record["value"], "1.2.3.4")

    def test_set_record_update_existing(self):
        """测试更新现有记录"""
        # 先创建一个记录
        self.provider.set_record("www~example.com", "1.2.3.4", "A")
        # 再更新它
        result = self.provider.set_record("www~example.com", "9.8.7.6", "A")
        self.assertTrue(result)
        record = self.provider._query_record("zone123", "www", "example.com", "A", None, {})
        if record:  # Type narrowing for mypy
            self.assertEqual(record["value"], "9.8.7.6")

    def test_set_record_invalid_domain(self):
        """测试无效域名"""
        result = self.provider.set_record("invalid.notfound", "1.2.3.4", "A")
        self.assertFalse(result)


if __name__ == "__main__":
    # 运行测试
    unittest.main(verbosity=2)



# coding=utf-8
"""
Unit tests for environment variable extra field support
@author: GitHub Copilot
"""

import os
from __init__ import unittest
from ddns.config.env import load_config  # noqa: E402


class TestEnvExtraFields(unittest.TestCase):
    """Test environment variable extra field parsing"""

    def setUp(self):
        """Clear DDNS environment variables before each test"""
        self._clear_env_prefix("DDNS_")

    def tearDown(self):
        """Clean up after tests"""
        self._clear_env_prefix("DDNS_")

    def _clear_env_prefix(self, prefix):
        # type: (str) -> None
        """Clear environment variables with a specific prefix (case-insensitive)"""
        keys_to_delete = [key for key in os.environ.keys() if key.upper().startswith(prefix.upper())]
        for key in keys_to_delete:
            del os.environ[key]

    def test_env_extra_single_field(self):
        """Test single DDNS_EXTRA_XXX environment variable"""
        os.environ["DDNS_DNS"] = "cloudflare"
        os.environ["DDNS_EXTRA_PROXIED"] = "true"

        config = load_config()
        self.assertEqual(config.get("dns"), "cloudflare")
        self.assertEqual(config.get("extra_proxied"), "true")

    def test_env_extra_multiple_fields(self):
        """Test multiple DDNS_EXTRA_XXX environment variables"""
        os.environ["DDNS_DNS"] = "alidns"
        os.environ["DDNS_EXTRA_PROXIED"] = "true"
        os.environ["DDNS_EXTRA_COMMENT"] = "Test comment"
        os.environ["DDNS_EXTRA_PRIORITY"] = "10"

        config = load_config()
        self.assertEqual(config.get("dns"), "alidns")
        self.assertEqual(config.get("extra_proxied"), "true")
        self.assertEqual(config.get("extra_comment"), "Test comment")
        self.assertEqual(config.get("extra_priority"), "10")

    def test_env_extra_with_standard_vars(self):
        """Test DDNS_EXTRA_XXX mixed with standard environment variables"""
        os.environ["DDNS_DNS"] = "cloudflare"
        os.environ["DDNS_ID"] = "test@example.com"
        os.environ["DDNS_TOKEN"] = "secret123"
        os.environ["DDNS_EXTRA_CUSTOM_FIELD"] = "custom_value"
        os.environ["DDNS_TTL"] = "300"

        config = load_config()
        self.assertEqual(config.get("dns"), "cloudflare")
        self.assertEqual(config.get("id"), "test@example.com")
        self.assertEqual(config.get("token"), "secret123")
        self.assertEqual(config.get("ttl"), "300")
        self.assertEqual(config.get("extra_custom_field"), "custom_value")

    def test_env_extra_case_insensitive(self):
        """Test that DDNS_EXTRA_XXX is case-insensitive"""
        os.environ["ddns_extra_field1"] = "value1"
        os.environ["DDNS_EXTRA_FIELD2"] = "value2"
        os.environ["Ddns_Extra_Field3"] = "value3"

        config = load_config()
        self.assertEqual(config.get("extra_field1"), "value1")
        self.assertEqual(config.get("extra_field2"), "value2")
        self.assertEqual(config.get("extra_field3"), "value3")

    def test_env_extra_with_underscores(self):
        """Test DDNS_EXTRA_XXX with underscores in field name"""
        os.environ["DDNS_EXTRA_CUSTOM_FIELD_NAME"] = "value1"
        os.environ["DDNS_EXTRA_ANOTHER_FIELD"] = "value2"

        config = load_config()
        self.assertEqual(config.get("extra_custom_field_name"), "value1")
        self.assertEqual(config.get("extra_another_field"), "value2")

    def test_env_extra_with_dots(self):
        """Test DDNS_EXTRA.XXX format (dots converted to underscores)"""
        os.environ["DDNS_EXTRA.FIELD1"] = "value1"
        os.environ["DDNS_EXTRA.FIELD2"] = "value2"

        config = load_config()
        # Dots should be converted to underscores
        self.assertEqual(config.get("extra_field1"), "value1")
        self.assertEqual(config.get("extra_field2"), "value2")

    def test_env_extra_numeric_values(self):
        """Test DDNS_EXTRA_XXX with numeric values"""
        os.environ["DDNS_EXTRA_PRIORITY"] = "100"
        os.environ["DDNS_EXTRA_WEIGHT"] = "0.5"

        config = load_config()
        self.assertEqual(config.get("extra_priority"), "100")
        self.assertEqual(config.get("extra_weight"), "0.5")

    def test_env_extra_empty_value(self):
        """Test DDNS_EXTRA_XXX with empty value"""
        os.environ["DDNS_EXTRA_COMMENT"] = ""

        config = load_config()
        self.assertEqual(config.get("extra_comment"), "")

    def test_env_no_extra_vars(self):
        """Test that config works without any extra environment variables"""
        # Clear all DDNS env vars first
        self._clear_env_prefix("DDNS_")

        os.environ["DDNS_DNS"] = "cloudflare"
        os.environ["DDNS_ID"] = "test@example.com"

        config = load_config()
        self.assertEqual(config.get("dns"), "cloudflare")
        self.assertEqual(config.get("id"), "test@example.com")
        # No extra_* keys should exist (only from this test)
        extra_keys = [k for k in config.keys() if k.startswith("extra_")]
        # Should have no extra keys from this test's environment variables
        self.assertEqual(len(extra_keys), 0, "Found unexpected extra keys: {}".format(extra_keys))

    def test_env_extra_json_array(self):
        """Test DDNS_EXTRA_XXX with JSON array format"""
        os.environ["DDNS_EXTRA_TAGS"] = '["tag1", "tag2", "tag3"]'

        config = load_config()
        self.assertEqual(config.get("extra_tags"), ["tag1", "tag2", "tag3"])

    def test_env_extra_special_characters(self):
        """Test DDNS_EXTRA_XXX with special characters"""
        os.environ["DDNS_EXTRA_URL"] = "https://example.com/path?key=value&foo=bar"

        config = load_config()
        self.assertEqual(config.get("extra_url"), "https://example.com/path?key=value&foo=bar")


if __name__ == "__main__":
    unittest.main()



# coding=utf-8
"""
DebugProvider
仅打印出 IP 地址，不进行任何实际 DNS 更新。
"""

from ._base import SimpleProvider


class DebugProvider(SimpleProvider):
    def _validate(self):
        """无需任何验证"""
        pass

    def set_record(self, domain, value, record_type="A", ttl=None, line=None, **extra):
        self.logger.debug("DebugProvider: %s(%s) => %s", domain, record_type, value)
        ip_type = "IPv4" if record_type == "A" else "IPv6" if record_type == "AAAA" else record_type
        print("[{}] {}".format(ip_type, value))
        return True



# coding=utf-8
"""
Unit tests for ddns.util.comment module
@author: GitHub Copilot
"""

from __future__ import unicode_literals
from __init__ import unittest
from ddns.util.comment import remove_comment


class TestRemoveComment(unittest.TestCase):
    """Test cases for comment removal functionality"""

    def test_remove_comment_empty_string(self):
        """测试空字符串"""
        result = remove_comment("")
        self.assertEqual(result, "")

    def test_remove_comment_no_comments(self):
        """测试没有注释的内容"""
        content = '{"key": "value", "number": 123}'
        result = remove_comment(content)
        self.assertEqual(result, content)

    def test_remove_comment_hash_full_line(self):
        """测试整行 # 注释"""
        content = '# This is a comment\n{"key": "value"}'
        expected = '\n{"key": "value"}'
        result = remove_comment(content)
        self.assertEqual(result, expected)

    def test_remove_comment_double_slash_full_line(self):
        """测试整行 // 注释"""
        content = '// This is a comment\n{"key": "value"}'
        expected = '\n{"key": "value"}'
        result = remove_comment(content)
        self.assertEqual(result, expected)

    def test_remove_comment_hash_with_leading_whitespace(self):
        """测试带前导空白的 # 注释"""
        content = '   # This is a comment\n{"key": "value"}'
        expected = '\n{"key": "value"}'
        result = remove_comment(content)
        self.assertEqual(result, expected)

    def test_remove_comment_double_slash_with_leading_whitespace(self):
        """测试带前导空白的 // 注释"""
        content = '  // This is a comment\n{"key": "value"}'
        expected = '\n{"key": "value"}'
        result = remove_comment(content)
        self.assertEqual(result, expected)

    def test_remove_comment_hash_end_of_line(self):
        """测试行尾 # 注释"""
        content = '{"key": "value"} # this is a comment'
        expected = '{"key": "value"}'
        result = remove_comment(content)
        self.assertEqual(result, expected)

    def test_remove_comment_double_slash_end_of_line(self):
        """测试行尾 // 注释"""
        content = '{"key": "value"} // this is a comment'
        expected = '{"key": "value"}'
        result = remove_comment(content)
        self.assertEqual(result, expected)

    def test_remove_comment_hash_in_string_should_not_remove(self):
        """测试字符串内的 # 不应该被移除"""
        content = '{"url": "http://example.com#anchor"}'
        result = remove_comment(content)
        self.assertEqual(result, content)

    def test_remove_comment_double_slash_in_string_should_not_remove(self):
        """测试字符串内的 // 不应该被移除"""
        content = '{"url": "http://example.com/path"}'
        result = remove_comment(content)
        self.assertEqual(result, content)

    def test_remove_comment_single_quoted_string_with_hash(self):
        """测试单引号字符串内的 # 不应该被移除"""
        content = "{'url': 'http://example.com#anchor'}"
        result = remove_comment(content)
        self.assertEqual(result, content)

    def test_remove_comment_single_quoted_string_with_double_slash(self):
        """测试单引号字符串内的 // 不应该被移除"""
        content = "{'url': 'http://example.com/path'}"
        result = remove_comment(content)
        self.assertEqual(result, content)

    def test_remove_comment_escaped_quotes_in_string(self):
        """测试字符串内转义引号的处理"""
        content = '{"message": "He said \\"Hello#World\\""} # comment'
        expected = '{"message": "He said \\"Hello#World\\""}'
        result = remove_comment(content)
        self.assertEqual(result, expected)

    def test_remove_comment_complex_json_with_comments(self):
        """测试复杂JSON配置与多种注释"""
        content = """{
    // Configuration file for DDNS
    "$schema": "https://ddns.newfuture.cc/schema/v4.0.json", // Schema validation
    "debug": false,  # false=disable, true=enable
    "dns": "dnspod_com",  // DNS provider
    "id": "1008666",      # ID or Email
    "token": "ae86$cbbcctv666666666666666",  // API Token or Key
    "ipv4": ["test.lorzl.ml"],  # IPv4 domains to update
    "ipv6": ["test.lorzl.ml"],  // IPv6 domains to update
    "index4": "public",     # IPv4 update method
    "index6": "url:https://iptest.com",  # IPv6 update method
    "proxy": null  // Proxy settings
}"""
        expected = """{

    "$schema": "https://ddns.newfuture.cc/schema/v4.0.json",
    "debug": false,
    "dns": "dnspod_com",
    "id": "1008666",
    "token": "ae86$cbbcctv666666666666666",
    "ipv4": ["test.lorzl.ml"],
    "ipv6": ["test.lorzl.ml"],
    "index4": "public",
    "index6": "url:https://iptest.com",
    "proxy": null
}"""
        result = remove_comment(content)
        self.assertEqual(result, expected)

    def test_remove_comment_mixed_comment_styles(self):
        """测试混合注释风格"""
        content = """// Header comment
{
    # This is a hash comment
    "key1": "value1", // End of line comment
    "key2": "value2"  # Another end of line comment
}
# Footer comment"""
        expected = """
{

    "key1": "value1",
    "key2": "value2"
}
"""
        result = remove_comment(content)
        self.assertEqual(result, expected)

    def test_remove_comment_comments_with_special_chars(self):
        """测试包含特殊字符的注释"""
        content = """// Comment with 中文字符 and émojis 🚀
{
    "test": "value" # Comment with symbols !@#$%^&*()
}"""
        expected = """
{
    "test": "value"
}"""
        result = remove_comment(content)
        self.assertEqual(result, expected)

    def test_remove_comment_preserve_empty_lines(self):
        """测试保留空行"""
        content = """// Comment

{
    "key": "value"
}

// Another comment"""
        expected = """

{
    "key": "value"
}

"""
        result = remove_comment(content)
        self.assertEqual(result, expected)

    def test_remove_comment_url_with_hash_and_comment(self):
        """测试URL中包含#，行尾有注释的情况"""
        content = '{"url": "https://example.com#section"} # This is a comment'
        expected = '{"url": "https://example.com#section"}'
        result = remove_comment(content)
        self.assertEqual(result, expected)

    def test_remove_comment_json_array_with_comments(self):
        """测试JSON数组与注释"""
        content = """[
    // First item
    "item1", # Comment 1
    "item2", // Comment 2
    "item3"  # Last item
]"""
        expected = """[

    "item1",
    "item2",
    "item3"
]"""
        result = remove_comment(content)
        self.assertEqual(result, expected)

    def test_remove_comment_nested_quotes(self):
        """测试嵌套引号的处理"""
        content = """{"message": "She said: \\"Don't use // or # here\\""} // comment"""
        expected = """{"message": "She said: \\"Don't use // or # here\\""} """
        result = remove_comment(content)
        # Note: we expect a trailing space where the comment was removed
        self.assertEqual(result, expected.rstrip())

    def test_remove_comment_multiple_slashes(self):
        """测试多个斜杠的情况"""
        content = '{"path": "C:\\\\Program Files\\\\App"} // Windows path'
        expected = '{"path": "C:\\\\Program Files\\\\App"}'
        result = remove_comment(content)
        self.assertEqual(result, expected)

    def test_remove_comment_hash_after_double_slash_comment(self):
        """测试 // 注释中包含 # 的情况"""
        content = '{"key": "value"} // Comment with # symbol'
        expected = '{"key": "value"}'
        result = remove_comment(content)
        self.assertEqual(result, expected)

    def test_remove_comment_single_line_various_formats(self):
        """测试单行多种格式"""
        test_cases = [
            ('{"key": "value"}', '{"key": "value"}'),  # No comment
            ("# Full line comment", ""),  # Full line hash
            ("// Full line comment", ""),  # Full line double slash
            ("   # Indented comment", ""),  # Indented hash
            ("   // Indented comment", ""),  # Indented double slash
            ('{"a": "b"} # End comment', '{"a": "b"}'),  # End hash
            ('{"a": "b"} // End comment', '{"a": "b"}'),  # End double slash
        ]

        for i, (input_content, expected) in enumerate(test_cases):
            result = remove_comment(input_content)
            self.assertEqual(
                result,
                expected,
                "Failed for test case %d: %r -> expected %r, got %r" % (i, input_content, expected, result),
            )


if __name__ == "__main__":
    unittest.main()



# -*- coding:utf-8 -*-
"""
ddns Package
"""

__description__ = "automatically update DNS records to my IP [域名自动指向本机IP]"

# 编译时，版本会被替换
__version__ = "4.1.2b1"

# 时间也会被替换掉
build_date = "2025-11-05T07:25:35Z"
